#include <qapplication.h>
#include <qcursor.h>
#include <qpixmap.h>
#include <qstyle.h>
#include <util/application.h>
#include <util/message.h>
#include <util/messagequeue.h>
#include <gui/desktop.h>
#include <qmutex.h>

QApplication* qApp = NULL;
QDesktopWidget* g_pcDesktopWidget = NULL;

enum
{
	ID_QT_EVENT = 100
};


class QApplicationPrivate
{
public:
	QStyle m_cStyle;
};

class QSyllableEvent
{
public:
	QSyllableEvent( QObject* pcReceiver, QEvent* pcEvent )
	{
		m_pcReceiver = pcReceiver;
		m_pcEvent = pcEvent;
		m_bDeleted = false;
	}
	~QSyllableEvent()
	{
		delete( m_pcEvent );
	}
	QObject* GetReceiver() { return( m_pcReceiver ); }
	QEvent* GetEvent() { return( m_pcEvent ); }
	bool GetDeleted() { return( m_bDeleted ); }
	void SetDeleted() { m_bDeleted = true; }
private:
	bool m_bDeleted;
	QObject* m_pcReceiver;
	QEvent* m_pcEvent;
};

QApplication::QApplication( int /*argc*/, char **/*argv*/, const char */*name*/ ) : QObject( NULL, "qapplication" ), os::Handler( "QApplication" )
{
	d = new QApplicationPrivate;
	g_pcDesktopWidget = new QDesktopWidget();
	qApp = this;
	
	os::Application* pcApp = os::Application::GetInstance();
	if( pcApp == NULL )
	{
		printf( "QApplication created with no os::Application!\n" );
		return;
	}
	pcApp->AddHandler( this );
}

QApplication::~QApplication()
{
	os::Application* pcApp = os::Application::GetInstance();
	if( pcApp  )
		pcApp->RemoveHandler( this );
	delete( g_pcDesktopWidget );
	g_pcDesktopWidget = NULL;
	delete( d );
}

void QApplication::enter_loop()
{
	printf("QApplication::enter_loop() not implemented!\n");
}

void QApplication::exit_loop()
{
}

QSize QApplication::globalStrut()
{
	return( QSize( 1, 1 ) );
}

QWidget* QApplication::focusWidget()
{
	/* Wee cannot implement this */
	printf("QApplication::focusWidget() called\n" );
	return( NULL );
}

int QApplication::startDragDistance()
{
	return( 20 );
}

void QApplication::setOverrideCursor( QCursor )
{
}

void QApplication::restoreOverrideCursor()
{
}

QPalette QApplication::palette( const QWidget* )
{
	return( QPalette( QColor( 255, 255, 255 ) ) );
}

QDesktopWidget* QApplication::desktop()
{
	return( g_pcDesktopWidget );
}

bool QApplication::sendEvent( QObject *pcReceiver, QEvent *pcEvent )
{
	if( pcReceiver == NULL )
	{
		printf( "QApplication::sendEvent: Unexpected null receiver" );
		return( false );
	}
	
	return( pcReceiver->event( pcEvent ) );
}
void QApplication::postEvent( QObject *pcReceiver, QEvent *pcEvent )
{
	if( pcReceiver == NULL )
	{
		printf( "QApplication::sendEvent: Unexpected null receiver" );
		return;
	}
	//printf("Post event to %s (%i)\n", pcReceiver->name(), pcEvent->type() );
	GlobalMutex::Lock();
	/* It seems that this doesn’t always work... */
	#if 1
	QSyllableEvent* pcSyllableEvent = new QSyllableEvent( pcReceiver, pcEvent );
	os::Message* pcMsg = new os::Message( ID_QT_EVENT );
	pcMsg->AddPointer( "qevent", pcSyllableEvent );
	os::Application::GetInstance()->PostMessage( pcMsg, qApp );
	#else
	pcReceiver->event( pcEvent );
	delete( pcEvent );
	#endif
	GlobalMutex::Unlock();	
}

void QApplication::sendPostedEvents( QObject *receiver, int nType )
{
	printf("QApplication::sendPostedEvents\n");
}

void QApplication::removePostedEvents( QObject *receiver )
{

	//printf("Removing events for %s\n", receiver->name() );
	GlobalMutex::Lock();
	os::Looper* pcApp = os::Application::GetInstance();
	
	
	pcApp->SpoolMessages();
	os::MessageQueue* pcQueue = pcApp->GetMessageQueue();
	pcQueue->Lock();
	os::Message* pcMsg;
	//printf("Message queue has %i messages\n", pcQueue->GetMessageCount() );
	
	
	for( int nIndex = 0; ( pcMsg = pcQueue->FindMessage( ID_QT_EVENT, nIndex ) ) != NULL; ++nIndex )
	{
		QSyllableEvent* pcSyllableEvent;
		if( pcMsg->FindPointer( "qevent", (void**)&pcSyllableEvent ) == 0 )
		{
			//printf( "Got QT event in queue!\n" );
			if( pcSyllableEvent->GetReceiver() == receiver )
			{
				printf("Got message for deleted object!\n" );
				pcSyllableEvent->SetDeleted();
			}
		} else
			printf("GOT POSTED QT EVENT WITHOUT POINTER!\n");
	}
	//printf("Message queue has %i messages\n", pcQueue->GetMessageCount() );
	pcQueue->Unlock();
	GlobalMutex::Unlock();
}


QStyle& QApplication::style() const
{
	return( d->m_cStyle );
}

bool QApplication::reverseLayout()
{
	return( false );
}
	

void QApplication::HandleMessage( os::Message* pcMessage )
{
	GlobalMutex::Lock();
	switch( pcMessage->GetCode() )
	{
		case ID_QT_EVENT:
		{
			QSyllableEvent* pcSyllableEvent;
			if( pcMessage->FindPointer( "qevent", (void**)&pcSyllableEvent ) == 0 )
			{
				if( !pcSyllableEvent->GetDeleted() )
				{
					//printf( "Fire event to %s %i!\n", pcSyllableEvent->GetReceiver()->name(), pcSyllableEvent->GetEvent()->type() );
					pcSyllableEvent->GetReceiver()->event( pcSyllableEvent->GetEvent() );
				} else {
					printf("Got deleted event %i!\n", pcSyllableEvent->GetEvent()->type() );
				}
				//printf("Fired!\n" );
				delete( pcSyllableEvent );
			} else
				printf("GOT QT EVENT with no ID!\n");
		}
		break;
		default:
			printf( "QApplication::HandleMessage() got unknown message\n" );
	}
	GlobalMutex::Unlock();
}

#include "qapplication.moc"








