
#include <qlistbox.h>
#include <gui/listview.h>
#include <util/message.h>

enum { ID_SELECTED };

class QListBox::Private
{
public:
    os::ListView* m_pcMenu;
    bool	  m_bIgnoreSelection;
};


QListBox::QListBox( QWidget* pcParent, const char *name ) : QWidget( pcParent, "list_box" )
{
    d = new Private;
    d->m_pcMenu = new os::ListView( GetBounds(), "list_box",
				    os::ListView::F_RENDER_BORDER | os::ListView::F_NO_HEADER | os::ListView::F_NO_AUTO_SORT | os::ListView::F_MULTI_SELECT,
				    os::CF_FOLLOW_ALL );
    d->m_pcMenu->InsertColumn( "", 150 );
    d->m_pcMenu->SetSelChangeMsg( new os::Message( ID_SELECTED ) );
    d->m_bIgnoreSelection = false;
    AddChild( d->m_pcMenu );
    d->m_pcMenu->SetTarget( this );
}

QListBox::~QListBox()
{
    delete d;
}

uint QListBox::count() const
{
    return( d->m_pcMenu->GetRowCount() );
}

void QListBox::clear()
{
    d->m_pcMenu->Clear();
}

void QListBox::insertItem( const QString &text, int index )
{
    std::string cBuffer;

    if ( text.isEmpty() == false ) {
	cBuffer = text.ascii();
    }
    QListBoxText* pcText = new QListBoxText( text );
    pcText->m_pcBox = this;
    d->m_pcMenu->InsertRow( index, pcText );
}

void QListBox::insertItem( const QListBoxItem* pcItem, int index )
{
	const_cast<QListBoxItem*>(pcItem)->m_pcBox = this;
    d->m_pcMenu->InsertRow( index, const_cast<QListBoxItem*>(pcItem) );
}

QListBoxItem* QListBox::item( int index ) const
{
	if( index >= (int)d->m_pcMenu->GetRowCount() )
		return( NULL );
	return( static_cast<QListBoxItem*>(d->m_pcMenu->GetRow( index ) ) );
}

QListBoxItem* QListBox::firstItem()
{
	if( d->m_pcMenu->GetRowCount() < 1 )
		return( NULL );
	
	return( static_cast<QListBoxItem*>(d->m_pcMenu->GetRow( 0 ) ) );
}

void QListBox::setSelectionMode( SelectionMode eMode )
{
    d->m_pcMenu->SetMultiSelect( eMode != Single );
}

void QListBox::setSelected( int nIndex, bool bSelected )
{
    if ( d->m_bIgnoreSelection ) {
	return;
    }
    d->m_pcMenu->Select( nIndex, false, bSelected );
}

void QListBox::setCurrentItem( int index )
{
    d->m_pcMenu->SetCurrentRow( index );
}

bool QListBox::isSelected( int nIndex ) const
{
    return( d->m_pcMenu->IsSelected( nIndex ) );
}

int QListBox::index( const QListBoxItem* pcItem ) const
{
    int nCount = d->m_pcMenu->GetRowCount();
    for ( int i = 0 ; i < nCount ; ++i ) {
	if ( static_cast<QListBoxItem*>(d->m_pcMenu->GetRow( i )) == pcItem ) {
	    return( i );
	}
    }
    return( -1 );
}

QSize QListBox::sizeHint() const
{
//    os::Point cSize = d->m_pcMenu->GetPreferredSize( false );
    return( QSize( 100/*cSize.x*/, 100/*cSize.y*/ ) );
}

void QListBox::HandleMessage( os::Message* pcMessage )
{
    switch( pcMessage->GetCode() )
    {	
	case ID_SELECTED:
	{
	    int nCount = d->m_pcMenu->GetRowCount();
	    d->m_bIgnoreSelection = true;
	    for ( int i = 0 ; i < nCount ; ++i ) {
//		emit activated( i /*d->m_pcMenu->GetFirstSelected()*/ );
		emit pressed( static_cast<QListBoxItem*>(d->m_pcMenu->GetRow(i)) );
	    }
	    d->m_bIgnoreSelection = false;
	    break;
	}
	default:
	    QWidget::HandleMessage( pcMessage );
	    break;
    }
}


QListBox* QListBoxItem::listBox() const
{
	return( m_pcBox );
}

QListBoxItem* QListBoxItem::next() const
{
	int nIndex;
	if( m_pcBox == NULL )
		return( NULL );
	if( ( nIndex = m_pcBox->index( this ) ) < 0 )
		return( NULL );
	return( m_pcBox->item( nIndex + 1 ) );
}

QListBoxText::QListBoxText( const QString& cString )
{
    AppendString( cString.utf8().data() );
}

#include "qlistbox.moc"

