#include <stdio.h>
#include "qpixmap.h"
#include <qpaintdevicemetrics.h>
#include <qscrollview.h>
#include <qstyle.h>
#include <qfont.h>
#include <qevent.h>
#include <gui/window.h>
#include <gui/control.h>
#include <cassert>

class QWidgetPrivate
{
public:
	bool m_bIsTopLevelWidget;
    int m_nHideCount;
    QStyle m_cStyle;
    QRect m_cGeometry;
    QPalette m_cPalette;
};

QWidget::QWidget( QWidget* pcParent, const char* pzName ) : QObject( pcParent, pzName ), View( os::Rect( 0, 0, 200, 200 ),
								  pzName == NULL ? "null" : pzName, os::CF_FOLLOW_LEFT | os::CF_FOLLOW_TOP,
								  os::WID_WILL_DRAW | os::WID_FULL_UPDATE_ON_RESIZE )
{
	isWidget = TRUE;
    d = new QWidgetPrivate;
    printf("QWidget::QWidget(%s)\n", GetName().c_str() );
	d->m_bIsTopLevelWidget = false;
    d->m_nHideCount = 1;
    Show( false );
    
    if ( pcParent != NULL ) {
	QScrollView* pcSView = dynamic_cast<QScrollView*>(pcParent);
	if ( pcSView != NULL ) {
	    pcParent = pcSView->viewport();
	    if ( pcParent == NULL || ( os::String( pzName ) == "scrollbar" ) ) {
		pcParent = pcSView;
	    }
	}
	pcParent->AddChild( this );
    }
}

QWidget::~QWidget()
{
    delete d;
}

QWidget* QWidget::parentWidget()
{
	if( GetParent() == NULL )
		return( NULL );
	QWidget* pcWidget = dynamic_cast<QWidget*>( GetParent() );
	return( pcWidget );
}

void QWidget::Lock()
{
	if( GetWindow() )
		assert( GetWindow()->Lock() == 0 );
}

void QWidget::Unlock()
{
	if( GetWindow() )
		assert( GetWindow()->Unlock() == 0 );
}

void QWidget::clearFocus()
{
    MakeFocus( false );
}

void QWidget::show()
{
    if ( d->m_nHideCount == 1 ) {
    Lock();
	Show( true );
	Unlock();
	d->m_nHideCount = 0;
    }
}

void QWidget::hide()
{
    if ( d->m_nHideCount == 0 ) {
   	Lock();
	Show( false );
	Unlock();
	d->m_nHideCount = 1;
    }
}


bool QWidget::isVisible() const
{
	return( d->m_nHideCount == 0 );
}

void QWidget::raise()
{
	MakeFocus();    
}

void QWidget::setFocusProxy( QWidget * )
{
//    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
}

bool QWidget::focusNextPrevChild( bool/* next*/ )
{
    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
    return( false );
}

QWidget* QWidget::focusWidget() const
{
	if( GetWindow() )
	{
		return( static_cast<QWidget*>(GetWindow()->GetFocusChild() ) );
	}
    return( NULL );
}

QPoint QWidget::mapToGlobal( const QPoint& cPos ) const
{
    os::Point cScrPos = ConvertToScreen( os::Point( cPos.x(), cPos.y() ) );
    return( QPoint( (int)cScrPos.x, (int)cScrPos.y ) );
}

QPoint QWidget::mapFromGlobal( const QPoint& cScrPos )
{
    os::Point cPos = ConvertFromScreen( os::Point( cScrPos.x(), cScrPos.y() ) );
    return( QPoint( (int)cPos.x, (int)cPos.y ) );
}

QSize QWidget::sizeHint() const
{
    os::Point cSize = GetPreferredSize( false );

    return( QSize( (int)cSize.x, (int)cSize.y ) );
}

void QWidget::setFont( const QFont &cFont )
{
	if( cFont.GetFont() )
		SetFont( cFont.GetFont() );
}

QFont QWidget::font() const
{
	printf( "QWidget::font() not implemented" );
	return( QFont() );
}

QStyle& QWidget::style() const
{
	return( d->m_cStyle );
}
    
    
int QWidget::x()
{
    return( int(GetFrame().left) );
}

int QWidget::y()
{
    return( int(GetFrame().top) );
}
    
int QWidget::width()
{
    return( int(GetFrame().Width()) + 1 );
}

int QWidget::height()
{
    return( int(GetFrame().Height()) + 1 );
}

QPoint QWidget::pos() const
{
    os::Rect cFrame = GetFrame();
    return( QPoint( (int)cFrame.left, (int)cFrame.top ) );
}

void QWidget::move( const QPoint& cPos )
{
    MoveTo( cPos.x(), cPos.y() );
}

QSize QWidget::size() const
{
    os::Rect cFrame = GetFrame();
    return( QSize( (int)cFrame.Width() + 1, (int)cFrame.Height() + 1 ) );
}

const QRect& QWidget::geometry() const
{
	d->m_cGeometry = QRect( pos(), size() );
	return( d->m_cGeometry );
}

int QWidget::frameWidth()
{
	return( 0 );
}

int QWidget::frameHeight()
{
	return( 0 );
}

void QWidget::resize( int w, int h )
{
	Lock();
    ResizeTo( w - 1, h - 1 );
    Unlock();
}

void QWidget::resize( const QSize& cSize )
{
    ResizeTo( cSize.width() - 1, cSize.height() - 1 );
}

void QWidget::setEraseColor( QColor cColor )
{
	SetEraseColor( os::Color32_s( cColor.red(), cColor.green(), cColor.blue() ) );
}

void QWidget::erase()
{
    FillRect( GetBounds(), os::Color32_s( 255, 255, 255 ) );
    Flush();
}

const QPalette& QWidget::palette() const
{
    return( d->m_cPalette );
}

void QWidget::setPalette( QPalette cPalette )
{
	d->m_cPalette = cPalette;
}

void QWidget::unsetPalette()
{
}
    
QWidget* QWidget::topLevelWidget()
{
    QWidget* pcParent = this;
    for (;;) {
    if( pcParent->IsTopLevelWidget() )
    	break;
	QWidget* pcTmp = dynamic_cast<QWidget*>(pcParent->GetParent());
	if ( pcTmp == NULL ) {
	    break;
	}
	pcParent = pcTmp;
    }
    printf( "DEBUG: Returning toplevel widget %s\n", pcParent->GetName().c_str() );
    return( pcParent );
}

void QWidget::setActiveWindow()
{
	if( GetWindow() )
		GetWindow()->MakeFocus();
}

bool QWidget::hasFocus()
{
	printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
    return( true );
}

void QWidget::setFocus()
{
    MakeFocus( true );
}

bool QWidget::isTopLevel() const
{
	return( IsTopLevelWidget() );
}

bool QWidget::isEnabled() const
{
    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
    return( true );
}

bool QWidget::isDefault() const
{
	return( HasFocus() );
}
bool QWidget::autoDefault() const
{
	return( false );
}

void QWidget::setMouseTracking( bool )
{
//    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
}

void QWidget::setBackgroundMode( BackgroundMode eMode )
{
	if( eMode != NoBackground )
	    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
}

void QWidget::setCursor( const QCursor& )
{
//    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
}

int QWidget::focusPolicy()
{
	return( ClickFocus );
}
    
void QWidget::setFocusPolicy( FocusPolicy )
{
//    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
}

void QWidget::setAcceptDrops( bool /*on*/ )
{
//    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
}

void QWidget::focusInEvent( QFocusEvent * )
{
}

void QWidget::focusOutEvent( QFocusEvent * )
{
}

void QWidget::resizeEvent ( QResizeEvent * event )
{
}

void QWidget::showEvent ( QShowEvent * )
{
}

void QWidget::hideEvent ( QHideEvent *)
{
}

void QWidget::closeEvent ( QCloseEvent *)
{
}

void QWidget::setEnabled( bool/* bEnabled*/ )
{
//    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
}

void QWidget::setAutoMask(bool)
{
//    printf( "Warning: QWidget::%s() not implemented\n", __FUNCTION__ );
}

void QWidget::repaint()
{
    Invalidate();
    Flush();
}

QFontMetrics QWidget::fontMetrics()
{
	QFontMetrics m( GetFont() );
	return( m );
}

int QWidget::metric( int id ) const
{
    switch( id )
    {
	case QPaintDeviceMetrics::PdmWidth:
	    return( int(GetFrame().Width()) );
	case QPaintDeviceMetrics::PdmHeight:
	    return( int(GetFrame().Height()) );
	case QPaintDeviceMetrics::PdmWidthMM:
	    printf( "Warning: QWidget::metric( PdmWidthMM ) not implemented\n" );
	    return( 200 );
	case QPaintDeviceMetrics::PdmHeightMM:
	    printf( "Warning: QWidget::metric( PdmHeightMM ) not implemented\n" );
	    return( 200 );
	case QPaintDeviceMetrics::PdmNumColors:
	    printf( "Warning: QWidget::metric( PdmNumColors ) not implemented\n" );
	    return( 65536 );
	case QPaintDeviceMetrics::PdmDepth:
	    printf( "Warning: QWidget::metric( PdmDepth ) not implemented\n" );
	    return( 16 );
	case QPaintDeviceMetrics::PdmDpiX:
	    return( 72 );
	case QPaintDeviceMetrics::PdmDpiY:
	    return( 72 );
	default:
	    printf( "Error: QWidget::metric( %d ) unknown ID\n", id );
	    return( -1 );
    }
}

void QWidget::FrameSized( const os::Point& cDelta )
{
    os::View::FrameSized( cDelta );
	os::Rect cNew = GetBounds();
	QResizeEvent cEvent( QSize( (int)cNew.Width() + 1, (int)cNew.Height() + 1 ), 
						QSize( (int)cNew.Width() + 1 - (int)cDelta.x,
						(int)cNew.Height() + 1 - (int)cDelta.y ) );
	resizeEvent( &cEvent );
}

void QWidget::Activated( bool bIsActive )
{
	if( bIsActive )
	{
		QFocusEvent cEvent( QEvent::FocusIn );
		focusInEvent( &cEvent );
	} else {
		QFocusEvent cEvent( QEvent::FocusOut );
		focusInEvent( &cEvent );
	}
}


os::Point QWidget::GetPreferredSize( bool bLargest ) const
{
    return( (bLargest) ? os::Point( 1000000.0f, 1000000.0f ) : os::Point( 0.0f, 0.0f ) );
}

bool QWidget::IsTopLevelWidget() const
{
	return( d->m_bIsTopLevelWidget );
}

void QWidget::SetTopLevelWidget( bool bSet )
{
	d->m_bIsTopLevelWidget = bSet;
}

#include "qwidget.moc"
#include "qstyle.moc"

























