#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "loader.h"
#include <kio/job.h>
#include <kdebug.h>
#include <kmdcodec.h>

#include <atheos/time.h>
#include <gui/bitmap.h>
#include <gui/guidefines.h>
#include <util/message.h>
#include <util/string.h>
#include <util/locker.h>
#include <util/resources.h>

#include <storage/file.h>
#include <storage/registrar.h>
#include <translation/translator.h>

#include <vector>
#include <list>
#include <map>
#include <algorithm>

#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

#include "../../kio/http/kcookiejar/kcookiejar.cpp"


#include "html/html_documentimpl.h"
#include "css/css_stylesheetimpl.h"
#include "xml/dom_docimpl.h"

#include <qmutex.h>
#include <cassert>

#define min( a, b ) (((a) < (b)) ? (a) : (b))


os::Bitmap* load_icon( os::StreamableIO* pcStream )
{
    os::Translator* pcTrans = NULL;

	os::TranslatorFactory* pcFactory = os::TranslatorFactory::GetDefaultFactory();
    
    os::Bitmap* pcBitmap = NULL;
    
    try {
	uint8  anBuffer[8192];
	uint   nFrameSize = 0;
	int    x = 0;
	int    y = 0;
	
	os::BitmapFrameHeader sFrameHeader;
	for (;;) {
	    int nBytesRead = pcStream->Read( anBuffer, sizeof(anBuffer) );

	    if (nBytesRead == 0 ) {
		break;
	    }

	    if ( pcTrans == NULL ) {
		int nError = pcFactory->FindTranslator( "", os::TRANSLATOR_TYPE_IMAGE, anBuffer, nBytesRead, &pcTrans );
		if ( nError < 0 ) {
		    return( NULL );
		}
	    }
	    
	    pcTrans->AddData( anBuffer, nBytesRead, nBytesRead != sizeof(anBuffer) );

	    while( true ) {
	    if ( pcBitmap == NULL ) {
		os::BitmapHeader sBmHeader;
		if ( pcTrans->Read( &sBmHeader, sizeof(sBmHeader) ) != sizeof( sBmHeader ) ) {
		    break;
		}
		pcBitmap = new os::Bitmap( sBmHeader.bh_bounds.Width() + 1, sBmHeader.bh_bounds.Height() + 1, os::CS_RGB32 );
	    }
	    if ( nFrameSize == 0 ) {
		if ( pcTrans->Read( &sFrameHeader, sizeof(sFrameHeader) ) != sizeof( sFrameHeader ) ) {
		    break;
		}
		x = sFrameHeader.bf_frame.left * 4;
		y = sFrameHeader.bf_frame.top;
		nFrameSize = sFrameHeader.bf_data_size;
	    }
	    uint8 pFrameBuffer[8192];
	    nBytesRead = pcTrans->Read( pFrameBuffer, min( nFrameSize, sizeof(pFrameBuffer) ) );
	    if ( nBytesRead <= 0 ) {
		break;
	    }
	    nFrameSize -= nBytesRead;
	    uint8* pDstRaster = pcBitmap->LockRaster();
	    int nSrcOffset = 0;
	    for ( ; y <= sFrameHeader.bf_frame.bottom && nBytesRead > 0 ; ) {
		int nLen = min( nBytesRead, sFrameHeader.bf_bytes_per_row - x );
		memcpy( pDstRaster + y * pcBitmap->GetBytesPerRow() + x, pFrameBuffer + nSrcOffset, nLen );
		if ( x + nLen == sFrameHeader.bf_bytes_per_row ) {
		    x = 0;
		    y++;
		} else {
		    x += nLen;
		}
		nBytesRead -= nLen;
		nSrcOffset += nLen;
	    }
	    }
	}
	if( pcTrans )
		delete pcTrans;
	return( pcBitmap );
    }
    catch( ... ) {
	return( NULL );
    }
}


using namespace DOM;

#define MAX_THREADS 6


KCookieJar g_cCookieJar;

namespace khtml {

class Request;


class LoaderLock
{
public:
    void Lock() { GlobalMutex::Lock(); }
    void Unlock() { GlobalMutex::Unlock(); }
};


//static sem_id		      g_hJobQueue;
static LoaderLock	      g_cLoaderLock;
static std::list<Request*>    g_cPendingRequests;
static std::list<Request*>    g_cRequests;
static atomic_t	      g_nWorkerThreadCount;
static atomic_t	      g_nRunningRequestCount;

    

    
class Request
{
public:
    Request( os::String cURL ) { m_pcDl = NULL; m_cURL = cURL, atomic_set( &m_nRefCount, 1 ); m_nLoadError = 0; m_bDone = false; m_bCanceled = false; m_pcObject = NULL; m_bIsProcessing = false; m_bIncremental = false; }
    virtual ~Request() {}
    virtual bool Run() = 0;
    virtual bool RunInput() = 0;
    virtual void HandleError( int nError ) = 0;
    virtual void Stop() = 0;

    virtual void Error( int nError ) { m_nLoadError = nError; }
    void SetIncremental( bool bInc ) { m_bIncremental = bInc; }
    
    void AddRef() { atomic_add( &m_nRefCount, 1 ); }
    void Release() {
	if ( atomic_dec_and_test( &m_nRefCount ) == 1 ) {
	    assert( m_bIsProcessing == false );

	    std::list<Request*>::iterator i;

	    g_cLoaderLock.Lock();
	    i = std::find( g_cPendingRequests.begin(), g_cPendingRequests.end(), this );
	    if ( i != g_cPendingRequests.end() ) {
		g_cPendingRequests.erase( i );
	    }
	    g_cLoaderLock.Unlock();
	    
	    delete this;
	}
    }
    
    void AddData( const char* pData, int nLen, bool /*bLast*/ ) {
	m_cInBuffer.insert( m_cInBuffer.end(), pData, pData + nLen );
    }
    void AddMetaData( const std::string& cMetaData ) {
	m_cMetaDataList.push_back( cMetaData );
    }

    std::vector<char>	m_cInBuffer;
    std::vector<std::string> m_cMetaDataList;
    bool		m_bDone;
    bool		m_bCanceled;

	os::String m_cURL;
    CachedObject*  m_pcObject;
    DocLoader* m_pcDl;

    bool m_bIsProcessing;
    bool m_bIncremental;
protected:    
   
    int		m_nLoadError;
private:    

    atomic_t m_nRefCount;
    
};

class HTTPLoader : public Request
{
public:
    HTTPLoader( const KURL& cURL );
    virtual ~HTTPLoader() { if ( m_nSocket != -1 ) close( m_nSocket ); }
    virtual bool Run();
    virtual bool RunInput();
    virtual void Stop();

    virtual void DataReceived( const char* pzData, int nLen, bool bFinal ) = 0;
    virtual void MetadataReceived( const std::string& cKey, const std::string& cValue ) = 0;
    virtual void MimetypeReceived( const std::string& /*cType*/ ) {}
    virtual void Done() = 0;
    virtual void Redirect( const std::string& /*cLocation*/ ) {}

    void SetPostData( const char* pData, int nSize ) {
	m_cPostData.insert( m_cPostData.end(), pData, pData + nSize );
    }
protected:    
    KURL	m_cURL;
    os::String	m_cRequestBuffer;
    os::String	m_cClientRequestBuffer;
    QCString	m_cRecvCookieStr;
    int			m_nResponseCode;
    int		m_nSocket;
    bool	m_bIsRunning;
private:
    std::string 	m_cLocationStr;
    std::string		m_cMimeType;
    std::vector<char>	m_cPostData;
};



class FileRequest : public Request
{
public:
    FileRequest( CachedObject* pcObject, const char* pzPath );
    ~FileRequest() { m_pcObject->setRequest( 0 ); }
    virtual bool Run();
    virtual bool RunInput() { return( false ); }
    virtual void Stop() { m_bCanceled = true; m_pcObject->finish(); }
    virtual void HandleError( int nError ) { m_pcObject->error( nError, "" ); }
    QBuffer        m_cBuffer;
    os::File*      m_pcFile;
};

class TransfereJobFile : public KIO::TransferJob, public Request
{
public:
    TransfereJobFile( const char* pzPath );
    virtual ~TransfereJobFile() { delete m_pcFile; }
    virtual bool Run();
    virtual bool RunInput() { return( false ); }
    virtual void HandleError( int nError ) { m_errorText = m_cURL.c_str(); m_error = nError; emit result( this ); };

    virtual void Start() { m_bStarted = true; }
    virtual void Stop() { m_bCanceled = true; }
    virtual void putOnHold() { m_bStarted = false; }

    virtual void kill();

    virtual KURL url() { return( KURL( m_cURL.c_str() ) ); }
    virtual QString queryMetaData( const QString &key );
    
private:    
    os::File*      m_pcFile;
    bool	   m_bStarted;
    bool	   m_bDone;
    
};



class TransfereJobHTTP : public KIO::TransferJob, public HTTPLoader
{
public:
    TransfereJobHTTP( const KURL& cURL );
    virtual ~TransfereJobHTTP() {}
    
    virtual void Start();
    virtual void putOnHold();
    virtual void DataReceived( const char* pzData, int nLen, bool bFinal );
    virtual void MetadataReceived( const std::string& cKey, const std::string& cValue );
    virtual void MimetypeReceived( const std::string& cType );
    virtual void Redirect( const std::string& cLocation );
    virtual void Done();
    virtual void HandleError( int nError ) { m_errorText = m_cURL.url(); m_error = nError; emit result( this ); };

    
    virtual void kill();

    virtual KURL url() { return( m_cURL ); }
    
    virtual void    addMetaData( const QString &key, const QString &value );
    virtual void    addMetaData( const QMap<QString,QString> &values );
    virtual QString queryMetaData( const QString &key );
    
private:
    std::map<os::String,os::String>	m_cReceivedMetaData;
    bigtime_t			        m_nLastUpdateTime;
};

class HTTPRequest : public HTTPLoader
{
public:
    HTTPRequest( CachedObject* pcObject, const KURL& cURL /*const char* pzHost, const char* pzPath*/ );
    ~HTTPRequest() { m_pcObject->setRequest( 0 ); }
    
    virtual void Start();
    virtual void Stop() { HTTPLoader::Stop(); m_pcObject->finish(); }
    virtual void HandleError( int nError ) { m_pcObject->error( nError, "" ); }

    virtual void DataReceived( const char* pzData, int nLen, bool bFinal );
    virtual void MetadataReceived( const std::string& cKey, const std::string& cValue );
    virtual void Done();
private:;    
    QBuffer        m_cBuffer;
};


} // end of khtml namespace

using namespace khtml;

static int32 worker_thread( void* /*pData*/ )
{
    int nError;

    for (;;) {
#if 0    	
	nError = lock_semaphore( g_hJobQueue );
	if ( nError < 0 ) {
	    if ( errno == EINTR ) {
		continue;
	    } else {
		atomic_dec( &g_nWorkerThreadCount );
		break;
	    }
	}
#endif	
	Request* pcRequest = NULL;
	g_cLoaderLock.Lock();
	if ( g_cPendingRequests.size() > 0 ) {
	    pcRequest = *g_cPendingRequests.begin();
	    g_cPendingRequests.erase( g_cPendingRequests.begin() );
	} else {
	    g_cLoaderLock.Unlock();
	    //continue;
	    atomic_dec( &g_nWorkerThreadCount );
	    return( 0 );
	}
	pcRequest->m_bIsProcessing = true;
	pcRequest->AddRef();
	g_cLoaderLock.Unlock();

	atomic_inc( &g_nRunningRequestCount );
	pcRequest->RunInput();
	atomic_dec( &g_nRunningRequestCount );
	
	g_cLoaderLock.Lock();
	pcRequest->m_bIsProcessing = false;
	pcRequest->Release();

	bool bDoExit = false;
	if ( g_cPendingRequests.empty() /*&& atomic_read( &g_nWorkerThreadCount ) > 1*/ ) {
	    atomic_dec( &g_nWorkerThreadCount );
	    bDoExit = true;
	}

	g_cLoaderLock.Unlock();
	if ( bDoExit ) {
	    break;
	}
    }
    return( 0 );
}


void init_loader()
{
    static bool bIsInitiated = false;

    if ( bIsInitiated ) {
	return;
    }
    bIsInitiated = true;
    //g_hJobQueue = create_semaphore( "http_job_queue", 0, 0 );
	atomic_set( &g_nWorkerThreadCount, 0 );
	atomic_set( &g_nRunningRequestCount, 0 );

}


static void add_http_request( HTTPLoader* pcLoader )
{
    g_cLoaderLock.Lock();
    g_cPendingRequests.push_back( pcLoader );
    

    if ( atomic_read( &g_nWorkerThreadCount ) < MAX_THREADS && atomic_read( &g_nWorkerThreadCount ) < ((int)g_cPendingRequests.size() + atomic_read( & g_nRunningRequestCount )) ) {
	thread_id hTread = spawn_thread( "http_worker", (void*)worker_thread, 0, 0, NULL );
	if ( hTread >= 0 ) {
	    resume_thread( hTread );
	    atomic_inc( &g_nWorkerThreadCount );
	}
    }
    //unlock_semaphore( g_hJobQueue );
    g_cLoaderLock.Unlock();
}




HTTPLoader::HTTPLoader( const KURL& cURL ) : Request( cURL.url().ascii() )
{
    m_cMimeType = "text/html";
    m_cURL = cURL;
    m_nSocket = -1;
	m_nResponseCode = -1;
    m_bIsRunning = false;
}

bool HTTPLoader::RunInput()
{
    std::string cMetaDataBuffer;
    struct sockaddr_in sAddress;
    uint nBytesSendt = 0;
    char anBuffer[65536];
    bool bReadingHeaders = true;
    struct hostent* psHost = NULL;
    
    
    if ( m_nSocket != -1 ) {
	close( m_nSocket );
	m_nSocket = -1;
    }
    
    if ( m_cURL.host().ascii() != NULL ) {
	psHost = gethostbyname( m_cURL.host().ascii() );
    }
    if ( psHost == NULL ) {
	printf( "Error: FAILED TO LOOKUP '%s'\n", m_cURL.url().ascii() );
	m_bDone = true;
	Error( KIO::ERR_UNKNOWN_HOST );
	return( false );
    }

    m_nSocket = socket( AF_INET, SOCK_STREAM, 0 );

    if ( m_nSocket < 0 ) {
	Error( KIO::ERR_COULD_NOT_CREATE_SOCKET );
	m_bDone = true;
	return( false );
    }
    
    m_cRequestBuffer = os::String().Format( "%s %s HTTP/1.0\r\n", (m_cPostData.empty()) ? "GET" : "POST", m_cURL.encodedPathAndQuery( 0, true ).ascii() );
    m_cRequestBuffer += "User-Agent: Mozilla/5.0 (compatible; ABrowse 0.4; Syllable)\r\n";
    
    m_cRequestBuffer += os::String().Format( "Host: %s\r\n", m_cURL.host().ascii() );
    m_cRequestBuffer += "Connection: close\r\n";
    m_cRequestBuffer += m_cClientRequestBuffer;
    
    /* Add username and password if provided */
    if( m_cURL.hasUser() || m_cURL.hasPass() )
    {
    	printf( "Sending username and password\n");
    	QCString cAuth( m_cURL.user().utf8() );
    	cAuth += ':';
    	cAuth += m_cURL.pass().utf8();
	    //printf( "Add User %s Pass %s\n", m_cURL.user().ascii(), m_cURL.pass().ascii() );
    	m_cRequestBuffer += "Authorization: Basic ";
    	m_cRequestBuffer += KCodecs::base64Encode( cAuth ).data();
    	m_cRequestBuffer += "\r\n";
    }
    
    if ( m_cPostData.size() > 0 ) {
	m_cRequestBuffer += os::String().Format( "Content-Length: %d\r\n", m_cPostData.size() );
    }
    m_cRequestBuffer += "\r\n";

    memset( &sAddress, 0, sizeof( sAddress ) );

    sAddress.sin_family = AF_INET;
    if ( m_cURL.port() == 0 ) {
	sAddress.sin_port  = htons(80);
    } else {
	sAddress.sin_port  = htons(m_cURL.port());
    }
    memcpy( &sAddress.sin_addr, psHost->h_addr_list[0], psHost->h_length );

	char nOut[16];
	uint8* ip = (uint8*)&sAddress.sin_addr;
	sprintf( nOut, "%d.%d.%d.%d", (int)(ip)[0], (int)(ip)[1], (int)(ip)[2], (int)(ip)[3] );
	printf("IP %s -> %s:%i\n", m_cURL.host().ascii(), nOut, sAddress.sin_port );
    if ( connect( m_nSocket, (struct sockaddr*) &sAddress, sizeof(sAddress) ) < 0 ) {
	Error( KIO::ERR_COULD_NOT_CONNECT );
	goto done;
    }
	



    while( nBytesSendt < m_cRequestBuffer.size() ) {
	int nLen = write( m_nSocket, m_cRequestBuffer.c_str() + nBytesSendt, m_cRequestBuffer.size() - nBytesSendt );
	if ( nLen < 0 ) {
	    Error( KIO::ERR_CONNECTION_BROKEN );
	    goto done;
	}
	nBytesSendt += nLen;
    }
    m_cRequestBuffer.resize(0);

    nBytesSendt = 0;
    while( nBytesSendt < m_cPostData.size() ) {
	int nLen = write( m_nSocket, &m_cPostData[0] + nBytesSendt, m_cPostData.size() - nBytesSendt );
	if ( nLen < 0 ) {
	    Error( KIO::ERR_CONNECTION_BROKEN );
	    goto done;
	}
	nBytesSendt += nLen;
    }
    m_cPostData.resize(0);
    
    while( bReadingHeaders ) {
	int nLen = read( m_nSocket, anBuffer, 65536 );
	if ( nLen <= 0 ) {
	    if ( nLen < 0 ) {
		Error( KIO::ERR_CONNECTION_BROKEN );
	    }
	    goto done;
	}
	g_cLoaderLock.Lock();
	for ( int i = 0 ; i < nLen ; ++i ) {
	    if ( anBuffer[i] == '\r' ) {
		continue;
	    }
	    if ( anBuffer[i] == '\n' ) {
		if ( cMetaDataBuffer.size() > 0 ) {
		    AddMetaData( cMetaDataBuffer );
		    cMetaDataBuffer.resize( 0 );
		} else {
		    AddData( anBuffer + i + 1, nLen - i - 1, false );
		    bReadingHeaders = false;
		    break;
		}
	    } else {
		cMetaDataBuffer += anBuffer[i];
	    }
	}
	g_cLoaderLock.Unlock();
    }
    for (;;) {
	int nLen = read( m_nSocket, anBuffer, 65536 );
	if ( nLen <= 0 ) {
	    if ( nLen < 0 ) {
		Error( KIO::ERR_CONNECTION_BROKEN );
	    }
	    goto done;
	}
	g_cLoaderLock.Lock();
	AddData( anBuffer, nLen, false );
	g_cLoaderLock.Unlock();
    }
done:
    g_cLoaderLock.Lock();
    close( m_nSocket );
    m_nSocket = -1;
    g_cLoaderLock.Unlock();
    m_bDone = true;
    return( true );
}



bool HTTPLoader::Run()
{
    if ( m_bCanceled ) {
	return( false );
    }
    if ( m_nLoadError != 0 ) {
	HandleError( m_nLoadError );
	return( false );
    }
    	
    g_cLoaderLock.Lock();
    
    for ( uint i = 0 ; i < m_cMetaDataList.size() ; ++i ) {
    	
    /* Check for the response code */
    if( strncasecmp( m_cMetaDataList[i].c_str(), "HTTP/", 5 ) == 0 )
    {
		m_nResponseCode = atoi( m_cMetaDataList[i].c_str() + 9 );
    }
    
    /* Check meta data in the format Key:Value */	  
	size_t nIndex = m_cMetaDataList[i].find( ':' );

	if ( nIndex != std::string::npos ) {
	    os::String cKey( m_cMetaDataList[i], 0, nIndex );
	    os::String cValue( m_cMetaDataList[i], nIndex + 1, -1 );
	    
	    cKey.Strip();
	    cValue.Strip();
	
	      // Check for cookies
	    if ( cKey.CompareNoCase( "Set-Cookie" ) == 0 || cKey.CompareNoCase( "Set-Cookie2" ) == 0 ) {
		m_cRecvCookieStr += m_cMetaDataList[i].c_str();
		m_cRecvCookieStr += '\n';
	    } else if ( cKey.CompareNoCase( "Location" ) == 0 ) {
		m_cLocationStr = cValue;
	    } else if ( cKey.CompareNoCase( "Content-type" ) == 0 ) {
		m_cMimeType = cValue;
	    } else {
		MetadataReceived( cKey, cValue );
	    }
	}
    }
    m_cMetaDataList.clear();

    if ( m_cLocationStr.empty() ) {
	if ( m_cInBuffer.size() > 0 ) {
	    if ( m_cMimeType.empty() == false ) {
		MimetypeReceived( m_cMimeType );
		m_cMimeType.resize( 0 );
	    }
	    DataReceived( &m_cInBuffer[0], m_cInBuffer.size(), false );
	}
    }
    m_cInBuffer.resize( 0 );
    bool bDone = m_bDone;
    g_cLoaderLock.Unlock();

    if ( bDone ) {
	if (!m_cRecvCookieStr.isEmpty()) {
		
	      // Give cookies to the cookiejar.
	    KHttpCookieList list = g_cCookieJar.makeCookies( m_cURL.url(), m_cRecvCookieStr, 0 );

		printf("Adding %i cookies\n", list.count() );
		for( uint i = 0; i < list.count(); i++ )
		{
		 
		KHttpCookiePtr next_cookie = list.at( i );
		g_cCookieJar.addCookie(next_cookie);
	    }
	    m_cRecvCookieStr = "";
	    
	}
	
	if ( m_cLocationStr.empty() == false && m_nResponseCode > 299 && m_nResponseCode < 400 ) {
		printf("Redirect to %s\n", m_cLocationStr.c_str() ); 
	    KURL u( m_cURL/*orig*/, QString::fromUtf8( m_cLocationStr.c_str() ) );
	    m_cLocationStr = u.url().utf8().data();
	    Redirect( m_cLocationStr );
	    m_cURL = m_cLocationStr.c_str();
	    m_cLocationStr.resize( 0 );
	    m_bDone = false;
	    add_http_request( this );
	    return( true );
	}
	Done();
	return( false );
    } else {
	return( true );
    }
}


void HTTPLoader::Stop()
{
    m_bIsRunning = false;
    m_bCanceled  = true;

    Error( KIO::ERR_USER_CANCELED );
    
    std::list<Request*>::iterator i;

    g_cLoaderLock.Lock();
    int nSocket = m_nSocket;
    m_nSocket = -1;
    if ( nSocket != -1 ) {
	close( nSocket );
    }
    i = std::find( g_cPendingRequests.begin(), g_cPendingRequests.end(), this );
    if ( i != g_cPendingRequests.end() ) {
	g_cPendingRequests.erase( i );
    }
    g_cLoaderLock.Unlock();
}



TransfereJobFile::TransfereJobFile( const char* pzPath ) : Request( pzPath )
{
    m_bStarted = false;
    m_bDone    = false;
    m_pcFile = NULL;
    m_mimetype = "";
    
    try {
	m_pcFile = new os::File( pzPath );
    } catch( ... ) {
    	m_bDone = true;
    	Error( KIO::ERR_CANNOT_OPEN_FOR_READING );
    }
}

bool TransfereJobFile::Run()
{
    const int nBlockSize = 32768;

    if ( m_bCanceled ) {
	return( false );
    }
    if ( m_nLoadError != 0 ) {
	HandleError( m_nLoadError );
	return( false );
    }
    
    if ( m_bStarted == false ) {
	return( true );
    }
    if ( m_pcFile == NULL ) {
	emit result( this );
	return( false );
    }
    
    /* Find mimetype */
   
    if( m_mimetype == "" )
	{   
	    os::String zType;
    	os::String zMimeType = "text/html";
	    os::Message cMsg;
		os::RegistrarManager* pcManager = os::RegistrarManager::Get();
	    pcManager->GetTypeAndIcon( m_cURL, os::Point(), &zType, NULL, &cMsg );
    	if( cMsg.FindString( "mimetype", &zMimeType ) == 0 )
	    {
	    	if( zMimeType == "application/x-file" )
    			zMimeType = "text/html";
	    }
	    m_mimetype = QString::fromUtf8( zMimeType.c_str() );
    	emit mimetype( this, m_mimetype );
    	pcManager->Put();
	}

    
    char pBuffer[nBlockSize];
    int nBytesRead = m_pcFile->Read( pBuffer, nBlockSize );
    if ( nBytesRead == 0 ) {
	emit result( this );
	m_bDone = true;
	return( false );
    }
    
    QByteArray cBuffer( nBytesRead );

    memcpy( cBuffer.data(), pBuffer, nBytesRead );

    emit data( this, cBuffer );

    if ( nBytesRead != nBlockSize ) {
	emit result( this );
	m_bDone = true;
    }
    return( nBytesRead == nBlockSize );
}


QString TransfereJobFile::queryMetaData( const QString& key )
{
	if( key == "content-length" && m_pcFile )
	{
		char zContentLength[255];
		sprintf( zContentLength, "%i", m_pcFile->GetSize() );
		return( zContentLength );
	}
    return( QString( "" ) );
}

void TransfereJobFile::kill()
{
    m_bCanceled = true;
}


TransfereJobHTTP::TransfereJobHTTP( const KURL& cURL ) :
	HTTPLoader( cURL )
{
    QString cCookies = g_cCookieJar.findCookies( cURL.url(), false, 0 );
    if ( cCookies.isEmpty() == false ) {
	m_cClientRequestBuffer += cCookies.utf8().data();
	m_cClientRequestBuffer += "\r\n";
    }
    m_nLastUpdateTime = 0;
    m_bIsRunning = false;
}

void TransfereJobHTTP::DataReceived( const char* pData, int nLen, bool /*bFinal*/ )
{
	QByteArray cData(nLen);
	memcpy( cData.data(), pData, nLen );
	emit data( this, cData );
}

void TransfereJobHTTP::MetadataReceived( const std::string& cKey, const std::string& cValue )
{
    m_cReceivedMetaData[os::String(cKey).Lower()] = cValue;
}

void TransfereJobHTTP::Redirect( const std::string& cLocation )
{
    emit redirection( this, KURL( cLocation.c_str() ) );
}

void TransfereJobHTTP::MimetypeReceived( const std::string& cType )
{
	m_mimetype = QString::fromUtf8( cType.c_str() );
    emit mimetype( this, m_mimetype );
    
}

void TransfereJobHTTP::Done()
{
     emit result( this );
}


void TransfereJobHTTP::Start()
{
	m_bIsRunning = true;
	add_http_request( this );
}

void TransfereJobHTTP::putOnHold()
{
    m_bIsRunning = false;
}

void TransfereJobHTTP::kill()
{
    HTTPLoader::Stop();
}

void TransfereJobHTTP::addMetaData( const QString &key, const QString &value )
{
	printf("Add meta data %s %s\n", key.utf8().data(), value.utf8().data() );
    if ( key == "referrer" ) {
	  // HTTP uses "Referer" although the correct
	  // spelling is "referrer"
	m_cClientRequestBuffer += "Referer: "; //os::String().Format( "Referer: %s\r\n", value.utf8().data() );
	m_cClientRequestBuffer += value.utf8().data();
	m_cClientRequestBuffer += "\r\n";
    } else {
	if ( strncasecmp( key.utf8().data(), value.utf8().data(), key.utf8().size() - 1 ) == 0 ) {
		if( value.find( ':' ) < 0 )
			return;
	    m_cClientRequestBuffer += value.utf8().data();
	    m_cClientRequestBuffer += "\r\n";
	} else {
	    m_cClientRequestBuffer += os::String().Format( "%s: %s\r\n", key.utf8().data(), value.utf8().data() );
	}
    }
}

void TransfereJobHTTP::addMetaData( const QMap<QString,QString> &values )
{
    for ( QMap<QString,QString>::ConstIterator i = values.begin() ; i != values.end() ; ++i ) {
	addMetaData( i.key(), i.data() );
    }
}

QString TransfereJobHTTP::queryMetaData( const QString& key )
{
    QString cData = m_cReceivedMetaData[key.lower().ascii()].c_str();
    return( cData );
}


FileRequest::FileRequest( CachedObject* pcObject, const char* pzPath ) : Request( pzPath )
{
    m_pcObject = pcObject;
    pcObject->setRequest( this );
    m_cBuffer.open( IO_ReadWrite );
    m_pcFile = NULL;
    try {
	m_pcFile = new os::File( pzPath );
    } catch( ... ) {
    	m_bDone = true;
    	Error( KIO::ERR_CANNOT_OPEN_FOR_READING );
    }
}


bool FileRequest::Run()
{
    if ( m_bCanceled ) {
	return( false );
    }
    if ( m_nLoadError != 0 ) {
	HandleError( m_nLoadError );
	return( false );
    }    
    const int nBlockSize = 32768;
    if ( m_pcFile == NULL ) {
	return( false );
    }
    char pBuffer[nBlockSize];
    int nBytesRead = m_pcFile->Read( pBuffer, nBlockSize );
    if ( nBytesRead == 0 ) {
	return( false );
    }
    m_cBuffer.writeBlock( pBuffer, nBytesRead );
	if( m_bIncremental || ( nBytesRead != nBlockSize ) )
	    m_pcObject->data( m_cBuffer, nBytesRead != nBlockSize );
    if ( nBytesRead != nBlockSize ) {
	m_pcObject->finish();
    }
    return( nBytesRead == nBlockSize );
}


HTTPRequest::HTTPRequest( CachedObject* pcObject, const KURL& cURL ) :
	HTTPLoader( cURL )
{
    pcObject->setRequest( this );
    m_cBuffer.open( IO_ReadWrite );
    m_pcObject = pcObject;
}
    
void HTTPRequest::DataReceived( const char* pData, int nLen, bool bFinal )
{
    m_cBuffer.writeBlock( pData, nLen );
		
	if( m_bIncremental || bFinal )
	    m_pcObject->data( m_cBuffer, bFinal );
}

void HTTPRequest::MetadataReceived( const std::string& /*cKey*/, const std::string& /*cValue*/ )
{
}

void HTTPRequest::Done()
{
    m_pcObject->data( m_cBuffer, true );
    m_pcObject->finish();
}

void HTTPRequest::Start()
{
    m_bIsRunning = true;
    add_http_request( this );
}

KIO::SimpleJob* KIO::http_update_cache( const KURL &/*url*/, bool /*no_cache*/, time_t /*expireDate*/ )
{
/*
    KIO_ARGS << (int)2 << url << no_cache << expireDate;
    SimpleJob *job = new SimpleJob( url, CMD_SPECIAL, packedArgs, false );
    return job;*/
    return( NULL );
}


Loader::Loader()
{
}


void Loader::load( DocLoader* pcDl, CachedObject *object, bool incremental )
{
    KURL url( object->url().string().ascii() );

    if ( url.protocol() == "file" ) {
	FileRequest* pcRequest = new FileRequest( object, url.path().ascii() );
	pcRequest->m_pcDl = pcDl;
	pcRequest->SetIncremental( incremental );
	g_cRequests.push_back( pcRequest );
	emit requestStarted( pcDl, object );
    } else if ( url.protocol() == "http" ) {
	//printf( "Get HTML ELEMENT WITH HTTP '%s' : '%s'\n", url.host().ascii(), url.path().ascii() );

	HTTPRequest* pcRequest = new HTTPRequest( object, url/*, url.encodedPathAndQuery( 0, true ).ascii()*/ );
	pcRequest->m_pcDl = pcDl;
	pcRequest->SetIncremental( incremental );
//	printf( "START REQUEST\n" );
//	printf( "ADD REQUEST\n" );
	g_cRequests.push_back( pcRequest );
	pcRequest->Start();
//	printf( "REQUEST STARTED\n" );
	emit requestStarted( pcDl, object );
    } else {
	printf( "Error: Loader::load() unknown protocol '%s'\n", url.protocol().ascii() );
    }
}

int Loader::numRequests( DocLoader* pcDl ) const
{
    int nCount = 0;
    g_cLoaderLock.Lock();
    for ( std::list<Request*>::iterator i = g_cRequests.begin() ; i != g_cRequests.end() ; ++i ) {
 	    if( (*i)->m_pcDl == pcDl )
	    	nCount++;
    }
    g_cLoaderLock.Unlock();
    return( nCount );
}

//    int numRequests( const DOM::DOMString &baseURL, CachedObject::Type type ) const;

void Loader::cancelRequests( DocLoader* pcDl )
{
    int nCount = 0;
	g_cLoaderLock.Lock();
    for ( std::list<Request*>::iterator i = g_cRequests.begin() ; i != g_cRequests.end() ; ) {
	if ( (*i)->m_pcObject != NULL && /*(*i)->m_cURL == os::String( pcDl->doc()->URL().ascii() )*/
									(*i)->m_pcDl == pcDl ) {
        Cache::removeCacheEntry( (*i)->m_pcObject );
	    KIO::TransferJob* pcJob = dynamic_cast<KIO::TransferJob*>(*i);
	    if ( pcJob != NULL ) {
		pcJob->kill();
	    } else {
		(*i)->Stop();
		(*i)->Release();
	    }
	    i = g_cRequests.erase( i );
	    nCount++;
	} else {
	    ++i;
	}
    }
    g_cLoaderLock.Unlock();
}

void Loader::Run()
{
    for ( std::list<Request*>::iterator i = g_cRequests.begin() ; i != g_cRequests.end() ; ) {
	Request* pcRequest = *i;
	if ( pcRequest->Run() ) {
	    ++i;
	} else {
	    i = g_cRequests.erase( i );
	    if( pcRequest->m_pcObject )
	    	emit Cache::loader()->requestDone( (DocLoader*)pcRequest->m_pcDl, (CachedObject*)pcRequest->m_pcObject );
	    pcRequest->Release();
	}
    }
}


void KIO::Job::kill()
{
}

QString KIO::Job::errorString() const
{
	//return KIO::buildErrorString(m_error, m_errorText);
	return( m_errorText );
}

void KIO::Job::showErrorDialog( QWidget *pcParent )
{
	printf( "Job::showErrorDialog() not implemented\n" );
}

KIO::TransferJob::TransferJob()
{
	m_error = 0;
}

KIO::TransferJob::~TransferJob()
{
}



void KIO::TransferJob::Start()
{
}

void KIO::TransferJob::addMetaData( const QString& /*key*/, const QString& /*value*/ )
{
}

void KIO::TransferJob::addMetaData( const QMap<QString,QString>& /*values*/ )
{
}

QString KIO::TransferJob::queryMetaData( const QString& /*key*/ )
{
//    printf( "KIO::TransferJob::queryMetaData() : '%s'\n", key.ascii() );
    return( "" );
}

KIO::MetaData KIO::TransferJob::outgoingMetaData()
{
	KIO::MetaData cData;
	return( cData );
}

KIO::TransferJob* KIO::get( const KURL& url, bool /*reload*/, bool /*showProgressInfo*/ )
{
    init_loader();
    if ( url.protocol() == "file" ) {
    g_cLoaderLock.Lock();
	TransfereJobFile* pcRequest = new TransfereJobFile( url.path().ascii() );
	//pcRequest->Start();
	g_cRequests.push_back( pcRequest );
	g_cLoaderLock.Unlock();
	return( pcRequest );
    } else if ( url.protocol() == "http" ) {
	printf( "Get HTTP '%s' : '%s'\n", url.host().ascii(), url.encodedPathAndQuery( 0, true ).ascii() );
	g_cLoaderLock.Lock();
	TransfereJobHTTP* pcRequest = new TransfereJobHTTP( url );
	//pcRequest->Start();
	g_cRequests.push_back( pcRequest );
	g_cLoaderLock.Unlock();
	return( pcRequest );
    } else {
	printf( "Error: Unknown protocol '%s'\n", url.protocol().ascii() );
	g_cLoaderLock.Lock();
	TransfereJobFile* pcRequest = new TransfereJobFile( "blank.html" );
	g_cRequests.push_back( pcRequest );
	//pcRequest->Start();
	g_cLoaderLock.Unlock();
	return( pcRequest );
    }
}


KIO::TransferJob* KIO::http_post( const KURL& url, const QByteArray& data, bool /*showProgressInfo*/ )
{
    init_loader();
    printf( "POST HTTP '%s' (%s)\n", url.encodedPathAndQuery( 0, true ).ascii(), data.data() );

    g_cLoaderLock.Lock();
    TransfereJobHTTP* pcRequest = new TransfereJobHTTP( url );
    pcRequest->SetPostData( data.data(), data.size() );
    //pcRequest->Start();
    g_cRequests.push_back( pcRequest );
    g_cLoaderLock.Unlock();
    return( pcRequest );
}




#include "../../../dropin/kio/jobclasses.moc"









