#include "application.h"
#include "utils.h"

using namespace os;

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

//this is where the default path is set up
BrowserApp::BrowserApp( const char *pzName, int argc, char *argv[] ):Application( pzName )
{

	/* Register types */
	RegisterTypes();

	/* mutex wars */
	GlobalMutex::Lock();
	SetMutex( GlobalMutex::GetMutex() );
	
	/* start kde app */
	g_pcKApp = new KApplication( argc, argv, "abrowse" );



	try
	{
		os::Directory cDir( GetHomePath() );
		os::Directory cBookmarkDir;
		os::Directory cBookmarkToolBarDir;
		cDir.CreateDirectory( "Bookmarks", &cBookmarkDir, cDir.GetMode() );
		cDir.CreateDirectory( "Bookmarks/ToolBar",&cBookmarkToolBarDir,cDir.GetMode());
	}
	catch( ... )
	{
	}
	

	/* register html symbols */
	KLibLoader::self()->library( "kjs_html", KLibLoader::CreateStaticWrapper )->registerSymbol( "kjs_html_init", ( void * )&kjs_html_init );
	KLibLoader::self()->library( "libkhtml", KLibLoader::CreateStaticWrapper )->registerSymbol( "init_libkhtml", ( void * )&init_libkhtml );


	/* Load cookies */
	os::Path cCookieFile( GetHomePath() );
	cCookieFile.Append( "Settings/Abrowse/cookies" );
	g_cCookieJar.loadCookies( cCookieFile.GetPath().c_str() );

	

	BrowserWindow *pcWindow = new BrowserWindow( Rect( 50, 20, 600, 480 ),true );

	if (argc > 1)
	{
		pcWindow->OpenFromArgs(argv[1]);
	}
	else
		pcWindow->OpenDefault();




	pcWindow->Show();
	pcWindow->MakeFocus();
	AddTimer( this, 1, 50000, false );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

BrowserApp::~BrowserApp()
{
	/* Wait until all windows have closed */
	Lock();
	while( BrowserWindow::s_cWindows.size() > 0 )
	{
		Unlock();
		snooze( 10000 );
		Lock();
	}

	Unlock();

	khtml::Cache::statistics();

	os::Path cCookieFile( GetHomePath() );
	cCookieFile.Append( "Settings/Abrowse/cookies" );
	g_cCookieJar.saveCookies( cCookieFile.GetPath().c_str() );
	
	
	khtml::Cache::clear();
	khtml::CSSStyleSelector::clear();
	khtml::RenderStyle::cleanup();		

	delete( g_pcKApp );

	if( m_pcManager )
		m_pcManager->Put();
}

bool BrowserApp::OkToQuit()
{
	if( BrowserWindow::s_cWindows.size() == 0 )
		return( true );
	
	/* Delete window from list */
	Lock();

	for( uint i = 0; i < BrowserWindow::s_cWindows.size(); i++ ) {
		BrowserWindow::s_cWindows[i]->PostMessage( os::M_QUIT );
	}
	Unlock();
	return( false );
}

void BrowserApp::TimerTick( int /*nID */  )
{
	khtml::Loader::Run();
}

void BrowserApp::RegisterTypes()
{
	m_pcManager = NULL;
	try
	{
		m_pcManager = os::RegistrarManager::Get();
		
		m_pcManager->RegisterType( "text/x-bookmark", "Bookmark" );
		m_pcManager->RegisterTypeIconFromRes( "text/x-bookmark", "text_bookmark.png" );
		m_pcManager->RegisterAsTypeHandler( "text/x-bookmark" );
		
		m_pcManager->RegisterType( "text/html", "HTML Page" );
		m_pcManager->RegisterTypeExtension( "text/html", "htm" );
		m_pcManager->RegisterTypeExtension( "text/html", "html" );
		m_pcManager->RegisterTypeIconFromRes( "text/html", "text_html.png" );
		m_pcManager->RegisterAsTypeHandler( "text/html" );
	} 
	catch( ... ) 
	{
	}
}

