/*  Syllable Dock
 *  Copyright (C) 2003 Arno Klenke
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of version 2 of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */
 
#include <gui/menu.h>
#include <gui/image.h>
#include <storage/directory.h>
#include <storage/fsnode.h>
#include <storage/file.h>
#include <util/string.h>
#include <util/resources.h>
#include <gui/layoutview.h>

#include "bookmarkstoolbar.h"
#include "bookmarks.h"
#include "messages.h"
#include "utils.h"

#include <storage/registrar.h>
#include <queue>

BookmarksToolBar::BookmarksToolBar( os::Handler* pcHandler, os::Path cPath, os::Rect cFrame, const char* pzName, int nResizeMask )
	 	: os::ToolBar( cFrame, pzName, nResizeMask )
{
	m_cPath = cPath;
	m_pcHandler = pcHandler;
	
	/* Create a nodemonitor for this directory */
	m_pcMonitor = new os::NodeMonitor( cPath.GetPath(), NWATCH_ALL, pcHandler );
	
	m_pcOtherMenu = new os::Menu(os::Rect(),"other_menu",os::ITEMS_IN_COLUMN);	
	
	
	m_pcContextMenu = new os::Menu( os::Rect( 0, 0, 0, 0 ), "toolbar_context", os::ITEMS_IN_COLUMN );
	m_pcContextMenu->AddItem("Add this page",new os::Message(ID_BOOKMARK_TOOLBAR_ADD));
	m_pcContextMenu->AddItem("Manage...",new os::Message(ID_BOOKMARK_TOOLBAR_MANAGE));
	m_pcContextMenu->SetTargetForItems(pcHandler);
	SetContextMenu(m_pcContextMenu);
	
	Reload();
}

void BookmarksToolBar::Reload()
{
	std::vector<os::LayoutNode*> children = GetRoot()->GetChildList();
	if (children.size() > 0)
	{
		for (uint i=0; i<children.size(); i++)
		{
			GetRoot()->RemoveChild(children[i]);
		}
	}
			
	/* Check if the directory is valid */
	os::FSNode* pcNode = NULL;
	try
	{
		pcNode = new os::FSNode( m_cPath.GetPath() );
	} catch( ... ) {
		return;
	}
	if( !pcNode->IsValid() )
	{
		delete( pcNode );
		return;
	}
	delete( pcNode );
	
	/* Iterate through the directory */
	os::Directory cDir( m_cPath.GetPath() );
	cDir.Rewind();
	os::String zFile;
	
	/* Get the registrar manager */
	os::RegistrarManager* pcManager = NULL;
	try
	{
		pcManager = os::RegistrarManager::Get();
		os::String zTemp;
		pcManager->GetTypeAndIcon( GetBookMarkToolBarPath(), os::Point( 32, 32 ), &zTemp, &m_pcDirIcon );		
	} catch( ... )
	{
	}

	while( cDir.GetNextEntry( &zFile ) == 1 )
	{
		if( zFile == os::String( "Launcher1.cfg" ) || zFile == os::String( "." ) || zFile == os::String( ".." ) )
			continue;
		
		os::Path cFilePath = m_cPath;
		cFilePath.Append( zFile.c_str() );
		os::Image* pcItemIcon = NULL;
		
		/* Get icon */
		if( pcManager )
		{
			os::String zTemp;
			if( pcManager->GetTypeAndIcon( cFilePath, os::Point( 32, 32 ), &zTemp, &pcItemIcon ) < 0 )
			{
				continue;
			}
		}
		
		/* Validate entry */
		try {
			pcNode = new os::FSNode( cFilePath );
		} catch( ... ) {
			continue;
		}
		if( !pcNode->IsValid() )
		{
			delete( pcNode );
			continue;
		}
			
		if( pcNode->IsFile() )
		{
			/* Create an menu item */
			os::Message *pcMsg = new os::Message( ID_GO_BOOKMARK );
			pcMsg->AddString( "file/path", cFilePath.GetPath() );
			
			if (zFile.Length() > 20)
			{
				zFile = zFile.substr(0,20);
				zFile += "...";
			}
			pcItemIcon->SetSize(os::Point(16,16));
			m_pcOtherMenu->AddItem(zFile,pcMsg,"",pcItemIcon);
		}
		if( pcNode->IsDir() )
		{			
			/* Create another DirMenu */
			BookmarksMenu* pcMenu = new BookmarksMenu( m_pcHandler, cFilePath, os::Rect(), zFile.c_str(), os::ITEMS_IN_COLUMN );
			m_pcDirItems.push(make_pair(pcMenu,pcItemIcon) );
		}
		delete( pcNode );
	}
	

	if (m_pcDirItems.size() >0)
	{
		while(!m_pcDirItems.empty())
		{
			os::Menu* menu = m_pcDirItems.front().first;
			menu->SetTargetForItems(m_pcHandler);		
			os::Image* image = m_pcDirItems.front().second;
			os::String cName = os::String("_popup_") + menu->GetName();
			AddPopupMenu(cName,menu->GetLabel(),image,menu);
			m_pcDirItems.pop();
		}
	}
	
	m_pcOtherMenu->SetTargetForItems(m_pcHandler);

	if( pcManager )
	{
		AddPopupMenu("_other_menu","Other",m_pcDirIcon,m_pcOtherMenu);
	}
	
	if( pcManager )
		pcManager->Put();	
}

BookmarksToolBar::~BookmarksToolBar()
{
	GetRoot()->RemoveChild(FindNode("_other_menu"));
	delete( m_pcMonitor );
}












