#include "findpanel.h"


#include <util/application.h>

class FindPanel::Private
{
public:
	Private(os::Messenger* pcTarget, const os::String& cText, uint32 nFlags)
	{
		//create target if none
		if (pcTarget == NULL)
			m_pcTarget = new os::Messenger(os::Application::GetInstance());
		else
			m_pcTarget = pcTarget;
			
		m_cText = cText;
		m_nFlags = nFlags;
	}
	
	StringView* m_pcFindString;
	TextView* m_pcFindTextView;
	
	StringView* m_pcReplaceString;
	TextView* m_pcReplaceTextView;
	
	ImageButton* m_pcCloseButton;
	
	os::Button* m_pcFindNext;
	os::Button* m_pcFindPrev;
	os::Button* m_pcReplace;
	os::Button* m_pcReplaceAll;
	
	os::CheckBox* m_pcMatchCase;
	
	os::HLayoutNode* m_pcRoot;
	
	uint32 m_nFlags;
	os::Messenger* m_pcTarget;
	os::String m_cText;
};

FindPanel::FindPanel(const os::String& cName, os::Messenger* pcTarget,const os::String& cText,uint32 nFlags) : os::LayoutView(os::Rect(0,0,1,1),cName,NULL,CF_FOLLOW_TOP | CF_FOLLOW_BOTTOM,WID_FULL_UPDATE_ON_RESIZE)
{
	m = new Private(pcTarget,cText,nFlags);
	_Layout();
}

void FindPanel::HandleMessage(Message* pcMessage)
{
	switch (pcMessage->GetCode())
	{
		case FIND_NEXT:
		{	
			if (m->m_pcFindTextView->GetBuffer()[0] != "")
			{		
				os::Message cMsg(FIND_NEXT);
				cMsg.AddString("find",m->m_pcFindTextView->GetBuffer()[0]);
				m->m_pcTarget->SendMessage(&cMsg);
			}
			break;
		}
		
		case FIND_PREV:
		{
			if (m->m_pcFindTextView->GetBuffer()[0] != "")
			{
				os::Message cMsg(FIND_PREV);
				cMsg.AddString("find",m->m_pcFindTextView->GetBuffer()[0]);
				m->m_pcTarget->SendMessage(&cMsg);
			}
			break;
		}
		case REPLACE:
		{
			if (m->m_pcFindTextView->GetBuffer()[0] != "" && m->m_pcReplaceTextView->GetBuffer()[0] !="")
			{
				os::Message cMsg(REPLACE);
				cMsg.AddString("find",m->m_pcFindTextView->GetBuffer()[0]);
				cMsg.AddString("replace",m->m_pcReplaceTextView->GetBuffer()[0]);
				m->m_pcTarget->SendMessage(&cMsg);
			}
			break;
		}	
		case REPLACE_ALL:
		{
			if (m->m_pcFindTextView->GetBuffer()[0] != "" && m->m_pcReplaceTextView->GetBuffer()[0] !="")
			{
				os::Message cMsg(REPLACE_ALL);
				cMsg.AddString("find",m->m_pcFindTextView->GetBuffer()[0]);
				cMsg.AddString("replace",m->m_pcReplaceTextView->GetBuffer()[0]);
				m->m_pcTarget->SendMessage(&cMsg);
			}
			break;
		}
		case MATCH_CASE:
		{
			os::Message cMsg(MATCH_CASE);
			cMsg.AddBool("match",m->m_pcMatchCase->GetValue().AsBool());
			m->m_pcTarget->SendMessage(&cMsg);
				
			break;
		}	
		case CLOSE:
		{
			os::Message cMsg(CLOSE);
			m->m_pcTarget->SendMessage(&cMsg);			
			break;
		}
	}
}

void FindPanel::AllAttached()
{
	m->m_pcCloseButton->SetTarget(this);
	m->m_pcFindNext->SetTarget(this);
	m->m_pcFindPrev->SetTarget(this);
	
	if (m->m_nFlags == FIND_AND_REPLACE_LAYOUT)
	{
		m->m_pcReplace->SetTarget(this);
		m->m_pcReplaceAll->SetTarget(this);
		m->m_pcMatchCase->SetTarget(this);
	}
	SetTabs();
}

void FindPanel::_Layout()
{
	m->m_pcRoot = new os::HLayoutNode("find_root_node");
	m->m_pcRoot->SetVAlignment(os::ALIGN_LEFT);
	m->m_pcRoot->SetHAlignment(os::ALIGN_LEFT);
	
	os::Resources cCol( get_image_id() );
	os::ResStream * pcStream = cCol.GetResourceStream( "close.png" );
	os::BitmapImage * pcIcon = new os::BitmapImage( pcStream );
	delete( pcStream );
	os::HLayoutNode* closeNode = new os::HLayoutNode("close_node");
	closeNode->AddChild(m->m_pcCloseButton=new os::ImageButton(os::Rect(),"close","",new os::Message(CLOSE),pcIcon,ImageButton::IB_TEXT_BOTTOM,true,false,true));
	closeNode->ExtendMinSize(os::Point(16,16));
	closeNode->ExtendMaxSize(os::Point(16,16));
	closeNode->LimitMaxSize(os::Point(16,16));
	
	m->m_pcRoot->AddChild(closeNode);
	m->m_pcRoot->AddChild(new os::HLayoutSpacer("",2,2));
	


	os::HLayoutNode* pcFindNode = new os::HLayoutNode("find_node");
	pcFindNode->AddChild(m->m_pcFindString = new os::StringView(os::Rect(),"Find_string","Find:"));
	pcFindNode->AddChild(new os::HLayoutSpacer("",2,2));
	pcFindNode->AddChild(m->m_pcFindTextView = new TextView(Rect(),"","",CF_FOLLOW_ALL));
	m->m_pcFindTextView->SetMinPreferredSize(20,1);
	m->m_pcFindTextView->SetMaxPreferredSize(200,1);
	m->m_pcRoot->AddChild(pcFindNode);
	
	if (m->m_nFlags == FIND_AND_REPLACE_LAYOUT)
	{
		m->m_pcRoot->AddChild(new os::HLayoutSpacer("",2,2));
		os::HLayoutNode* pcReplaceNode = new os::HLayoutNode("replace_node");
		pcReplaceNode->AddChild(m->m_pcReplaceString = new os::StringView(os::Rect(),"Replace_String","Replace:"));
		pcReplaceNode->AddChild(m->m_pcReplaceTextView = new TextView(Rect(),"",""));
		m->m_pcReplaceTextView->SetMinPreferredSize(12,1);
		m->m_pcReplaceTextView->SetMaxPreferredSize(12,1);		
		m->m_pcRoot->AddChild(pcReplaceNode);
	}
	
	m->m_pcRoot->AddChild(new os::HLayoutSpacer("",2,2));
	os::HLayoutNode* buttonNode = new os::HLayoutNode("button_node");
	
	buttonNode->AddChild(m->m_pcFindNext = new os::Button(os::Rect(),"find_next","_Find Next",new os::Message(FIND_NEXT)));
	buttonNode->AddChild(new os::HLayoutSpacer("",2,2));
	m->m_pcFindNext->MakeFocus();
	buttonNode->AddChild(m->m_pcFindPrev = new os::Button(os::Rect(),"find_prev","Find Prev",new os::Message(FIND_PREV)));
	buttonNode->SameWidth("find_next","find_prev",NULL);
	buttonNode->SameHeight("find_next","find_prev",NULL);
	buttonNode->AddChild(new os::HLayoutSpacer("",5,5));
	
	if (m->m_nFlags == FIND_AND_REPLACE_LAYOUT)
	{
		buttonNode->AddChild(m->m_pcReplace = new os::Button(os::Rect(),"replace","Replace",new os::Message(REPLACE)));
		buttonNode->AddChild(new os::HLayoutSpacer("",2,2));
		buttonNode->AddChild(m->m_pcReplaceAll = new os::Button(os::Rect(),"replace_all","Replace All",new os::Message(REPLACE_ALL)));
		buttonNode->SameWidth("replace","replace_all",NULL);
		buttonNode->SameHeight("replace","replace_all",NULL);
	}
	buttonNode->AddChild(new os::HLayoutSpacer("",2,2));
	buttonNode->AddChild(m->m_pcMatchCase=new os::CheckBox(os::Rect(),"match_case","Match Case",new os::Message(MATCH_CASE)));
	m->m_pcRoot->AddChild(buttonNode);
	SetRoot(m->m_pcRoot);
	ResizeTo(m->m_pcRoot->GetPreferredSize(false));
}

os::Point FindPanel::GetPreferredSize(bool bSize) const
{
	return (m->m_pcRoot->GetPreferredSize(false));
}

void FindPanel::Focus()
{
	m->m_pcFindTextView->MakeFocus();
	m->m_pcFindTextView->Clear();
}

void FindPanel::SetTabs()
{
	GetWindow()->SetDefaultButton(m->m_pcFindNext);
	
	m->m_pcFindTextView->SetTabOrder(0);
	

	if (m->m_nFlags == FIND_AND_REPLACE_LAYOUT)
	{
		m->m_pcReplaceTextView->SetTabOrder(1);
	}

	m->m_pcFindNext->SetTabOrder(2);
	m->m_pcFindPrev->SetTabOrder(3);
	
	if (m->m_nFlags == FIND_AND_REPLACE_LAYOUT)
	{
		m->m_pcReplace->SetTabOrder(4);
		m->m_pcReplaceAll->SetTabOrder(5);
	}
	m->m_pcMatchCase->SetTabOrder(6);
}












