#include "mainwindow.h"

std::vector<BrowserWindow*> BrowserWindow::s_cWindows;


class PanelSeparator : public os::Separator
{
public:
	PanelSeparator(const os::Rect& cRect,const os::String& cName, os::orientation nFlags, os::Point cSize) : os::Separator(cRect,cName,nFlags)
	{
		m_cSize = cSize;
	}
	
	virtual os::Point GetPreferredSize(bool bFlag) const 
	{
		return m_cSize;
	}
private:
	os::Point m_cSize;
};

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
BrowserWindow::BrowserWindow( const Rect & cFrame, bool bLoadPos ):Window( cFrame, "abrowse_wnd", "ABrowse", 0 )
{
	Init(bLoadPos);
	LoadSettings();
	
	SetupMenus();
	SetupToolBar();
	SetupBookmarksToolBar();
	SetupTabBar();
	CreateNewTab();	
	SetupFindPanel();
	SetupStatusBar();
	
	
	m_pcSettingsDialog = new SettingsDialog(new os::Messenger(this),m_pcSettings);	
	
	//lay out the inital system
	Layout();
	
	//add frame and statusbar
	AddFrame();
	AddChild(m_pcStatusBar);	
	
	//register events
	RegisterEvents();
	
	//set new frame
	SetFrame( m_cFrame );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::Init(bool bSaveSize)
{
	m_bFindPanel = m_bMatchCase = false;
	m_bBookmarkToolBar = m_bMainToolBar = m_bStatusBar = true;
	m_bSaveSize = bSaveSize;
	m_pcSettingsDialog = NULL;
	
	m_cFrame = GetBounds();
	
	s_cWindows.push_back( this );

	GlobalMutex::Lock();
	SetMutex( GlobalMutex::GetMutex() );

	m_pcFrame = new LayoutView( os::Rect(0,0,GetBounds().Width(),GetBounds().Height()-20), "frame_view",NULL,CF_FOLLOW_ALL );
	
	
	

	
	/* Set Icon */
	os::Resources cCol( get_image_id() );
	os::ResStream * pcStream = cCol.GetResourceStream( "icon24x24.png" );
	os::BitmapImage * pcIcon = new os::BitmapImage( pcStream );
	delete( pcStream );
	SetIcon( pcIcon->LockBitmap() );
	delete( pcIcon );	
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::SetupMenus()
{
	//lets setup the main menu bar
	m_pcMenuBar = new Menu( Rect( 0, 0, 0, 0 ), "menu_bar", ITEMS_IN_ROW );
	Menu *pcAppMenu = new Menu( Rect( 0, 0, 0, 0 ), "Application", ITEMS_IN_COLUMN );

	pcAppMenu->AddItem( "About", new os::Message( ID_ABOUT ) );
	pcAppMenu->AddItem( new os::MenuSeparator() );
	pcAppMenu->AddItem( "Settings",new os::Message(M_SETTINGS_REQUESTED));
	pcAppMenu->AddItem( new os::MenuSeparator());
	pcAppMenu->AddItem( "Quit", new os::Message( M_QUIT ) );
	m_pcMenuBar->AddItem( pcAppMenu );

	Menu *pcFileMenu = new Menu( Rect( 0, 0, 0, 0 ), "File", ITEMS_IN_COLUMN );
	pcFileMenu->AddItem(GetNewMenu());
	pcFileMenu->AddItem(new MenuSeparator());
	pcFileMenu->AddItem(GetOpenMenu());	
	pcFileMenu->AddItem(GetCloseMenu());
	pcFileMenu->AddItem(new MenuSeparator());
	pcFileMenu->AddItem("Save Page As...",new os::Message(ID_SAVE_LINK));
	pcFileMenu->AddItem("Send Link...",new os::Message(ID_SEND_LINK));
	pcFileMenu->AddItem(new MenuSeparator());
	pcFileMenu->AddItem(GetPrintMenu());
	pcFileMenu->GetItemAt(6)->SetEnable(false);
	m_pcMenuBar->AddItem(pcFileMenu);
	
	Menu *pcEditMenu = new Menu( Rect( 0, 0, 0, 0 ), "Edit", ITEMS_IN_COLUMN );
	
	pcEditMenu->AddItem( "Copy", new os::Message( ID_COPY ) );
	pcEditMenu->AddItem( "Paste", new os::Message( ID_PASTE ) );
	pcEditMenu->AddItem( "Select All", new os::Message(ID_SELECT_ALL));
	m_pcMenuBar->AddItem( pcEditMenu );
	
	
	Menu *pcActionsMenu = new Menu( Rect( 0, 0, 0, 0 ), "Actions", ITEMS_IN_COLUMN );
	
	pcActionsMenu->AddItem("Stop", new os::Message(ID_STOP));
	pcActionsMenu->AddItem("Reload",new os::Message(ID_RELOAD));
	pcActionsMenu->AddItem(new os::MenuSeparator());	
	pcActionsMenu->AddItem(GetTextSizeMenu());
	m_pcMenuBar->AddItem( pcActionsMenu );		
	
	Menu *pcViewMenu = new Menu( Rect( 0, 0, 0, 0 ), "View", ITEMS_IN_COLUMN );
	
	pcViewMenu->AddItem(GetToolBarMenu());
	pcViewMenu->AddItem(new os::CheckMenu("StatusBar",new os::Message(ID_VIEW_STATUS_BAR),true));
	pcViewMenu->AddItem(new os::MenuSeparator());	
	pcViewMenu->AddItem("View Source",new os::Message(ID_VIEW_SOURCE));
	m_pcMenuBar->AddItem( pcViewMenu );	
	
	
	
	m_pcBookmarkMenu = new BookmarksMenu( this, os::Path( GetBookMarkPath() ), os::Rect(), "Bookmarks", os::ITEMS_IN_COLUMN );
	m_pcBookmarkMenu->SetTargetForItems( this );
	m_pcMenuBar->AddItem( m_pcBookmarkMenu );
	
	Menu *pcToolsMenu = new Menu( Rect( 0, 0, 0, 0 ), "Tools", ITEMS_IN_COLUMN );
	pcToolsMenu->AddItem("Read Mail",new os::Message(ID_TOOLS_READ_MAIL));
	pcToolsMenu->AddItem("New Mail Message",new os::Message(ID_TOOLS_COMPOSE_MAIL));
	pcToolsMenu->AddItem(new os::MenuSeparator());
	pcToolsMenu->AddItem("Downloads",NULL);
	pcToolsMenu->GetItemAt(1)->SetEnable(false);
	pcToolsMenu->GetItemAt(3)->SetEnable(false);
	m_pcMenuBar->AddItem( pcToolsMenu);
	
	
	Menu *pcHelpMenu = new Menu( Rect( 0, 0, 0, 0 ), "Help", ITEMS_IN_COLUMN );
	m_pcMenuBar->AddItem(pcHelpMenu);
	
	m_pcMenuBar->SetTargetForItems( this );	

	//setup the tab menu
	m_pcTabMenu = new Menu( Rect( 0, 0, 0, 0 ), "Tab", ITEMS_IN_COLUMN );
	m_pcTabMenu->AddItem( "New Tab", new os::Message( ID_NEW_TAB ) );
	m_pcTabMenu->AddItem( "Close Tab", new os::Message( ID_DELETE_TAB ) );	
	m_pcTabMenu->SetTargetForItems( this );
}

os::Menu* BrowserWindow::GetNewMenu()
{
	Menu *pcReturnMenu = new Menu( Rect( 0, 0, 0, 0 ), "New", ITEMS_IN_COLUMN );
	pcReturnMenu->AddItem("Window",new os::Message(ID_NEW_WINDOW));
	pcReturnMenu->AddItem("Tab",new os::Message( ID_NEW_TAB ));
	return pcReturnMenu;
}

os::Menu* BrowserWindow::GetOpenMenu()
{
	Menu *pcReturnMenu = new Menu( Rect( 0, 0, 0, 0 ), "Open", ITEMS_IN_COLUMN );
	pcReturnMenu->AddItem("Location...",new os::Message(ID_FILE_OPEN_LOCATION));
	pcReturnMenu->AddItem("File...",new os::Message(ID_FILE_OPEN_FILE));
	return pcReturnMenu;
}

os::Menu* BrowserWindow::GetCloseMenu()
{
	Menu *pcReturnMenu = new Menu( Rect( 0, 0, 0, 0 ), "Close", ITEMS_IN_COLUMN );
	pcReturnMenu->AddItem("Window",new os::Message(ID_CLOSE_WINDOW));
	pcReturnMenu->AddItem("Tab",new os::Message(ID_DELETE_TAB));
	return pcReturnMenu;
}



os::Menu* BrowserWindow::GetPrintMenu()
{
	Menu *pcReturnMenu = new Menu( Rect( 0, 0, 0, 0 ), "Print", ITEMS_IN_COLUMN );
	pcReturnMenu->AddItem("Page Setup...",new os::Message(ID_PAGE_SETUP));
	pcReturnMenu->AddItem("Print...",new os::Message(ID_PRINT));
	pcReturnMenu->AddItem("Print Preview...",new os::Message(ID_PRINT_PREVIEW));
	pcReturnMenu->SetEnable(false);	
	return pcReturnMenu;
}


os::Menu* BrowserWindow::GetToolBarMenu()
{
	Menu *pcReturnMenu = new Menu( Rect( 0, 0, 0, 0 ), "ToolBars", ITEMS_IN_COLUMN );
	pcReturnMenu->AddItem(new os::CheckMenu("Navigation ToolBar",new os::Message(ID_VIEW_SHOW_MAIN_TOOLBAR),m_bMainToolBar));
	pcReturnMenu->AddItem(new os::CheckMenu("Bookmarks ToolBar",new Message(ID_VIEW_SHOW_BOOKMARK),m_bBookmarkToolBar));
	return pcReturnMenu;
}

os::Menu* BrowserWindow::GetTextSizeMenu()
{
	Menu *pcReturnMenu = new Menu( Rect( 0, 0, 0, 0 ), "Text Size", ITEMS_IN_COLUMN );
	pcReturnMenu->AddItem("Increase",new os::Message(ID_TEXT_INCREASE));
	pcReturnMenu->AddItem("Decrease",new os::Message(ID_TEXT_DECREASE));
	pcReturnMenu->AddItem("Normal",new os::Message(ID_TEXT_NORMAL));
	pcReturnMenu->SetEnable(false);
	return pcReturnMenu;
}



//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::SetupToolBar()
{
	m_pcURLView = new AddressPanel(new os::Messenger(this),ID_URL_CHANGED);
	
	m_pcToolBar = new ToolBar( Rect( 0, 0, 0, 0 ), "tool_bar" , CF_FOLLOW_LEFT | CF_FOLLOW_RIGHT | CF_FOLLOW_TOP);
	m_pcToolBar->AddChild(AddButton( "back.png", "Back", new os::Message( ID_PREV_URL)),os::ToolBar::TB_FIXED_MINIMUM);
	m_pcToolBar->AddChild(AddButton( "forward.png", "Forward", new os::Message(ID_NEXT_URL)),os::ToolBar::TB_FIXED_MINIMUM);
	m_pcToolBar->AddChild(AddButton( "reload.png", "Reload", new os::Message( ID_RELOAD)),os::ToolBar::TB_FIXED_MINIMUM);
	m_pcToolBar->AddChild(AddButton( "stop.png", "Stop", new os::Message(ID_STOP)),os::ToolBar::TB_FIXED_MINIMUM);	
	m_pcToolBar->AddChild(AddButton( "home.png", "Home", new os::Message( ID_GO_HOME)),os::ToolBar::TB_FIXED_MINIMUM);
	m_pcToolBar->AddSeparator( "" );
	m_pcToolBar->AddChild(AddButton("find.png", "Find", new os::Message( ID_FIND )),os::ToolBar::TB_FIXED_MINIMUM);
	
	for (uint i=0; i<m_sToolBarButtons.size(); i++)
	{
		m_pcToolBar->AddChild(m_sToolBarButtons[i],os::ToolBar::TB_FIXED_MINIMUM);
	}
	
	os::Point cPoint(m_cFrame.Width()-m_pcToolBar->GetRoot()->GetPreferredSize(false).x-30,m_pcToolBar->GetPreferredSize(false).y-16);	

	m_pcURLView->SetPreferredSize(cPoint);
	m_pcToolBar->AddChild( m_pcURLView,os::ToolBar::TB_FIXED_MINIMUM );
	m_pcToolBar->AddChild(AddButton("go.png", "Go", new os::Message(ID_GO_ACTIVATED)),os::ToolBar::TB_FIXED_MINIMUM);

	EnableButton( "Back",false );
	EnableButton( "Forward",false );	
}


void BrowserWindow::SetupBookmarksToolBar()
{
	m_pcBookmarkToolBar = new BookmarksToolBar(this,os::Path(GetBookMarkToolBarPath()),Rect(), "tool_bar" , CF_FOLLOW_LEFT | CF_FOLLOW_RIGHT | CF_FOLLOW_TOP);
}

os::ImageButton* BrowserWindow::AddButton(const os::String& cIcon,const os::String& cName,os::Message* pcMessage)
{
	os::ImageButton* pcButton = new os::ImageButton(os::Rect(),cName,cName,pcMessage,LoadImageFromResource(cIcon));
	return pcButton;
}

void BrowserWindow::EnableButton(const os::String& cName,bool bEnable)
{
	os::LayoutNode* pcNode = m_pcToolBar->GetRoot()->FindNode(cName);
	if (pcNode)
	{
		os::ImageButton* pcButton = dynamic_cast<os::ImageButton*>(pcNode->GetView());
		if( pcButton)
		{
			pcButton->SetEnable( bEnable );
		}	
	}	
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::SetupStatusBar()
{
	m_pcStatusBar = new StatusBar( Rect(0,m_pcFrame->GetBounds().bottom,GetBounds().Width(),GetBounds().Height()), "status_bar",CF_FOLLOW_BOTTOM | CF_FOLLOW_LEFT | CF_FOLLOW_RIGHT);
	m_pcStatusBar->AddPanel("info","");

	m_pcProgressBar = new os::ProgressBar(os::Rect(),"progress");
	m_pcStatusBar->GetRoot()->AddChild(m_pcProgressBar);

}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::SetupFindPanel()
{
	m_pcFindPanel = new FindPanel("Find",new os::Messenger(this));
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::SetupTabBar()
{
	m_pcTabs = new os::TabView( Rect( 0, 0, 0, 0 ), "tabs", CF_FOLLOW_LEFT | CF_FOLLOW_TOP, WID_WILL_DRAW | WID_CLEAR_BACKGROUND );
	m_pcTabs->SetContextMenu(m_pcTabMenu);
	m_pcTabs->SetMessage( new Message( ID_TAB_CHANGED ) );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::Layout()
{
	VLayoutNode *pcRoot = new VLayoutNode( "root" );	

	pcRoot->AddChild( m_pcMenuBar );
	
	
	if (m_bMainToolBar)
		pcRoot->AddChild( m_pcToolBar,0);
	
	
	if (m_bBookmarkToolBar)
	{
		os::VLayoutNode* pcBookmarkLayout = new os::VLayoutNode("bookmark_layout");
		pcBookmarkLayout->SetVAlignment(ALIGN_LEFT);
		pcBookmarkLayout->SetHAlignment(ALIGN_LEFT);
		pcBookmarkLayout->AddChild(new PanelSeparator(os::Rect(),"",HORIZONTAL,os::Point(GetBounds().Width()*2,6)));
		pcBookmarkLayout->AddChild(m_pcBookmarkToolBar);
		pcBookmarkLayout->AddChild(new PanelSeparator(os::Rect(),"",HORIZONTAL,os::Point(GetBounds().Width()*2,6)));	
		pcRoot->AddChild(pcBookmarkLayout);
	}
	
	pcRoot->AddChild( m_pcTabs );
	
	if (m_bFindPanel)
	{
		os::VLayoutNode* pcFindLayout = new os::VLayoutNode("find_layout");
		pcFindLayout->SetVAlignment(ALIGN_LEFT);
		pcFindLayout->SetHAlignment(ALIGN_LEFT);
		pcFindLayout->AddChild(new PanelSeparator(os::Rect(),"",HORIZONTAL,os::Point(GetBounds().Width()*2,6)));
		pcFindLayout->AddChild(m_pcFindPanel);
		pcFindLayout->AddChild(new PanelSeparator(os::Rect(),"",HORIZONTAL,os::Point(GetBounds().Width()*2,6)));
		pcRoot->AddChild(pcFindLayout);
	}
	m_pcFrame->SetRoot( pcRoot );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::AddFrame()
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
	KHTMLView *pcView = hview->GetPart()->view(  );


	pcView->SetResizeMask( CF_FOLLOW_ALL );

	os::font_height sFontHeight;
	m_pcTabs->GetFontHeight( &sFontHeight );
	float vGlyphHeight = sFontHeight.ascender + sFontHeight.descender + sFontHeight.line_gap;
	float vTabHeight = ceil( vGlyphHeight * 1.2f + 6.0f );

	os::Rect cClientBounds = m_pcTabs->GetBounds();
	cClientBounds.Resize( 0.0f, 0.0f, -4.0f, -2.0f - vTabHeight );
	cClientBounds.Floor();
	pcView->SetFrame( cClientBounds );

	pcView->MakeFocus();


	pcView->show();
	hview->GetPart()->browserExtension(  )->SetCallback( this );
	m_pcURLView->SetTarget( this );	

	AddChild(m_pcFrame);
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::LoadSettings()
{
	/* Load settings */
	m_pcSettings = new ABrowse_Settings();
	
	
	if ( m_pcSettings->LoadSettings() && m_pcSettings->LoadFinished())
	{		
		m_cFrame = m_pcSettings->GetFrame();
		m_cDefaultPath = m_pcSettings->GetDefaultPath();
	}
	  
	else
	{
		CenterInScreen();
		m_cFrame = GetFrame();
		m_pcSettings->SetFrame(m_cFrame);
		
		m_cDefaultPath = GetDefaultPath();
		m_pcSettings->SetDefaultPath(m_cDefaultPath);
	}
		
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::RegisterEvents()
{
	m_pcOpenInNewTabEvent = os::Event::Register("os/Internet/Browser/OpenInNewTab","Opens a URL in new Tab",this,ID_NEW_TAB);
	m_pcOpenInNewWindowEvent = os::Event::Register("os/Internet/Browser/OpenInNewWindow","Opens a URL in new Window",this,ID_NEW_TAB);
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::OpenFromArgs(const os::String& argv)
{
	os::String cDefaultPath;
	bool bIsBookmark = false;

	/* Check if the parameter is a bookmark */
	os::String zPath( argv );
	os::Path cPath( argv );
	if( zPath.Length() > 5  && zPath.substr( 0, 4 ) == "http" )
		cDefaultPath = zPath;
	else
		cDefaultPath = cPath.GetPath();
		
	try 
	{
		char zAttrib[255];
		memset( zAttrib, 0, sizeof( zAttrib ) );
		os::File cFile( cDefaultPath );
		if( cFile.ReadAttr( "os::MimeType", ATTR_TYPE_STRING, zAttrib, 0, 255 ) > 0 )
			if( strcmp( zAttrib, "text/x-bookmark" ) == 0 )
				bIsBookmark = true;
	} 
	catch( ... )
	{
	}
		
	if( bIsBookmark )
	{
		os::Message* pcMsg = new os::Message( ID_GO_BOOKMARK );
		pcMsg->AddString( "file/path", cDefaultPath );
		PostMessage( pcMsg, this );
	}
	else
	{
		KParts::URLArgs cArgs;		
		OpenURL( cDefaultPath.str(), cArgs );	
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
void BrowserWindow::OpenDefault()
{	
	KParts::URLArgs cArgs;		
	OpenURL( m_cDefaultPath, cArgs );	
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
BrowserWindow::~BrowserWindow()
{
	/* Stop loading */
	for ( int i = 0; i < m_pcTabs->GetTabCount(); ++i )
	{
		HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
		hview->GetPart()->closeURL();
	}
	RemoveChild( m_pcFrame );
	delete( m_pcFrame );
	/* Delete window from list */
	Lock();
	for( uint i = 0; i < s_cWindows.size(); i++ ) {
		if( s_cWindows[i] == this )
		{
			s_cWindows.erase( s_cWindows.begin() + i );
			break;
		}
	}
	Unlock();	
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------
AddressPanel *BrowserWindow::GetURLView()
{
	return ( m_pcURLView );
}

KHTMLPart *BrowserWindow::GetPart()
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	return ( hview->GetPart() );
}

void BrowserWindow::SwitchTab()
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	m_pcURLView->SetCurrentString( hview->GetCurrentURL().c_str());
	SetTitle( m_pcTabs->GetTabTitle( i ) + " - ABrowse" );
	std::string cURL = m_pcURLView->GetCurrentString();

	KHTMLView *pcView = hview->GetPart()->view(  );

	pcView->SetResizeMask( CF_FOLLOW_ALL );

	os::font_height sFontHeight;
	m_pcTabs->GetFontHeight( &sFontHeight );
	float vGlyphHeight = sFontHeight.ascender + sFontHeight.descender + sFontHeight.line_gap;
	float vTabHeight = ceil( vGlyphHeight * 1.2f + 6.0f );

	os::Rect cClientBounds = m_pcTabs->GetBounds();
	cClientBounds.Resize( 0.0f, 0.0f, -4.0f, -2.0f - vTabHeight );
	cClientBounds.Floor();

	pcView->SetFrame( cClientBounds );

	pcView->MakeFocus();
	GoHistory( 0, true );

}



void BrowserWindow::CreateNewTab()
{
	HTMLView *hview = new HTMLView( m_pcTabs->GetBounds(), "html_tab", os::CF_FOLLOW_ALL );

	hview->GetPart()->browserExtension(  )->SetCallback( this );
	hview->GetPart()->view()->show();
	m_pcTabs->AppendTab( "  (Untitled)  ", hview );

}
KParts::ReadOnlyPart* BrowserWindow::CreateNewTab( const std::string & cURL, const KParts::URLArgs & cArgs )
{
	HTMLView *hview = new HTMLView( m_pcTabs->GetBounds(), "html_tab", os::CF_FOLLOW_ALL );

	hview->GetPart()->browserExtension(  )->SetCallback( this );
	KHTMLView *pcView = hview->GetPart()->view(  );
	pcView->show();
	m_pcTabs->AppendTab( "  (Untitled)  ", hview );
	hview->OpenURL( cURL, cArgs );

	return( hview->GetPart() );
}

void BrowserWindow::DeleteTab()
{
	uint i = m_pcTabs->GetSelection();
	int nt = m_pcTabs->GetTabCount();

	if ( nt > 1 )
	{
		delete( m_pcTabs->DeleteTab( i ) );
	}
}


void BrowserWindow::UpdateHistory()
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	hview->UpdateHistory();
}


void BrowserWindow::AddHistoryEntry()
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	hview->AddHistoryEntry();
	EnableButton( "Forward",false );
	EnableButton( "Back", true );
}

void BrowserWindow::GoHistory( int nStep, bool bButtonsOnly )
{

	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

	if ( !bButtonsOnly )
	{
		hview->GoHistory( nStep );
	}
	HistoryEntry *pcCurrent = hview->GetCurrentHistory();

	if ( pcCurrent != NULL )
	{
		m_pcURLView->SetCurrentString( pcCurrent->m_cURL.prettyURL().utf8().data());
	}
	if ( hview->GetHistoryPosition() == 0 )
	{
		EnableButton( "Back",false );
	}
	else
	{
		EnableButton( "Back", true );
	}
	if ( hview->GetHistoryPosition() >= int ( hview->GetHistoryEntries(  ).size(  ) ) - 1 )
	{
		EnableButton( "Forward", false );
	}
	else
	{
		EnableButton( "Forward", true );
	}
}

#if 0
void dump_message( os::Message * pcMsg )
{
	int nCount = pcMsg->CountNames();

	for ( int i = 0; i < nCount; ++i )
	{
		int nType;
		int nCount;

		pcMsg->GetNameInfo( pcMsg->GetName( i ).c_str(), &nType, &nCount );
		switch ( nType )
		{
		case T_STRING:
			{
				std::string cStr;
				pcMsg->FindString( pcMsg->GetName( i ).c_str(), &cStr );
				printf( "%s: '%s'\n", pcMsg->GetName( i ).c_str(), cStr.c_str(  ) );
				break;
			}
		default:
			printf( "%s: (Type=%d)\n", pcMsg->GetName( i ).c_str(), nType );
			break;

		}
	}
}
#endif

void BrowserWindow::OpenURL( const std::string & cURL, const KParts::URLArgs & cArgs )
{
	uint i = m_pcTabs->GetSelection();
	HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
	os::String cURLSet = cURL;
	m_pcURLView->SetCurrentString( cURLSet.c_str());
	if ( cURL.find( "#" ) == std::string::npos )
		m_pcTabs->SetTabTitle( i, "Loading..." );
	hview->OpenURL( cURL, cArgs );

	EnableButton("Stop",true);
}

void BrowserWindow::OpenURLRequest( const std::string & cURL, const KParts::URLArgs & cURLArgs )
{
	UpdateHistory();
	AddHistoryEntry();
	OpenURL( cURL, cURLArgs );
	UpdateHistory();
}

void BrowserWindow::OpenURLNotify()
{
	UpdateHistory();
	AddHistoryEntry();
}

void BrowserWindow::Completed()
{
	UpdateHistory();

	EnableButton("Stop",false);
}

void BrowserWindow::SetLocationBarURL( const std::string & cURL )
{
	m_pcURLView->SetCurrentString( cURL.c_str());
}

void BrowserWindow::CreateNewWindow( const std::string & cURL, const KParts::URLArgs & cURLArgs )
{
	os::Rect cFrame( 0, 0, 799, 599 );

	cFrame += GetNewWinPosition();

	BrowserWindow *pcNewWindow = new BrowserWindow( cFrame, false );

	if ( cURL.empty() == false )
	{
		pcNewWindow->AddHistoryEntry();
		pcNewWindow->OpenURL( cURL, cURLArgs );
	}
	pcNewWindow->Show( true );
	pcNewWindow->MakeFocus( true );
}

os::Point BrowserWindow::GetNewWinPosition()
{
	static Point cLastPos( 100, 100 );

	cLastPos.x += 30;
	cLastPos.y += 30;
	if ( cLastPos.x > 500 )
	{
		cLastPos.x = 100;
	}
	if ( cLastPos.y > 500 )
	{
		cLastPos.y = 100;
	}
	return ( cLastPos );
}

KParts::ReadOnlyPart * BrowserWindow::CreateNewWindow( const std::string & cURL, const KParts::URLArgs & cURLArgs, const KParts::WindowArgs & sWndArgs )
{
	if( sWndArgs.lowerWindow )
	{
		return( CreateNewTab( cURL, cURLArgs ) );
	}
	
	os::Rect cFrame( 0, 0, 799, 599 );

	Point cPos;

	if ( sWndArgs.x == -1 || sWndArgs.y == -1 )
	{
		cPos = GetNewWinPosition();
	}
	if ( sWndArgs.x != -1 )
	{
		cPos.x = sWndArgs.x;
	}
	if ( sWndArgs.y != -1 )
	{
		cPos.y = sWndArgs.y;
	}
	if ( sWndArgs.width != -1 )
	{
		cFrame.right = cFrame.left + sWndArgs.width - 1;
	}
	if ( sWndArgs.height != -1 )
	{
		cFrame.bottom = cFrame.top + sWndArgs.height - 1;
	}
	cFrame += cPos + os::Point( 20, 20 );


	BrowserWindow *pcNewWindow = new BrowserWindow( cFrame, false );

	if ( cURL.empty() == false )
	{
		pcNewWindow->AddHistoryEntry();
		pcNewWindow->OpenURL( cURL, cURLArgs );
	}
	pcNewWindow->Show( true );
	pcNewWindow->MakeFocus( true );
	return ( pcNewWindow->GetPart() );

}

void BrowserWindow::PopupMenu( KParts::BrowserExtension * /*pcSource */ , const std::string & cURL )
{
	m_pcPopupMenu = new os::Menu( Rect( 0, 0, 10, 10 ), "popup", ITEMS_IN_COLUMN );
	m_pcPopupMenu->SetCloseMessage( os::Message( ID_POPUP_CLOSED ) );
	m_pcPopupMenu->SetCloseMsgTarget( this );

	os::Message * pcMsg;
	m_pcPopupMenu->AddItem( new os::MenuItem( "New window", new os::Message( ID_NEW_WINDOW ) ) );
	m_pcPopupMenu->AddItem( new os::MenuItem( "Reload", new os::Message( ID_RELOAD ) ) );

	if ( cURL.empty() == false )
	{
		m_pcPopupMenu->AddItem( new os::MenuSeparator() );

		pcMsg = new os::Message( ID_OPEN_TAB );
		pcMsg->AddString( "url", cURL );
		m_pcPopupMenu->AddItem( new os::MenuItem( "Open link in new tab", pcMsg ) );

		pcMsg = new os::Message( ID_OPEN_LINK );
		pcMsg->AddString( "url", cURL );
		m_pcPopupMenu->AddItem( new os::MenuItem( "Open link in new window", pcMsg ) );

		pcMsg = new os::Message( ID_SAVE_LINK );
		pcMsg->AddString( "url", cURL );
		m_pcPopupMenu->AddItem( new os::MenuItem( "Save link as...", pcMsg ) );

		pcMsg = new os::Message( ID_COPY_LINK_LOCATION );
		pcMsg->AddString( "url", cURL );
		m_pcPopupMenu->AddItem( new os::MenuItem( "Copy link location", pcMsg ) );
	}

	m_pcPopupMenu->AddItem( new os::MenuSeparator() );
	m_pcPopupMenu->AddItem( new os::MenuItem( "Quit", new os::Message( M_QUIT ) ) );

	m_pcPopupMenu->SetTargetForItems( this );

	os::Point cMousePos;
	m_pcStatusBar->GetMouse( &cMousePos, NULL );
	m_pcPopupMenu->Open( m_pcStatusBar->ConvertToScreen( cMousePos ) );
}

void BrowserWindow::LoadingProgress( int nPercent )
{
	m_pcProgressBar->SetProgress( nPercent*0.01 );
}

void BrowserWindow::SpeedProgress( int /*bytesPerSecond */  )
{
}

void BrowserWindow::InfoMessage( const std::string & cString )
{
	os::String cText;
	if( cString.find( "<qt>" ) == 0 )
		cText = cString.substr( 4, cString.size() - 4 );
	else
		cText = cString;
		    	
	m_pcStatusBar->SetText( "info",cText );
}

void BrowserWindow::SetWindowCaption( KParts::ReadOnlyPart * pcPart, const std::string & cCaption )
{
	Lock();

	for ( int i = 0; i < m_pcTabs->GetTabCount(); ++i )
	{
		HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

		if ( pcPart == hview->GetPart() )
		{
			if ( ( int )m_pcTabs->GetSelection() == i )
				SetTitle( cCaption + " - ABrowse" );
			m_pcTabs->SetTabTitle( i, cCaption );
		}
	}

	Unlock();
}

void BrowserWindow::SetStatusBarText( const std::string & cString )
{
	os::String cText;
	if( cString.find( "<qt>" ) == 0 )
		cText = cString.substr( 4, cString.size() - 4 );
	else
		cText = cString;
		    	
	m_pcStatusBar->SetText( "info",cText );
}


void BrowserWindow::SelectionInfo( const std::string & cText )
{
	m_cSelectedText = cText;
}

bool BrowserWindow::BeginDownload( KParts::DownloadStream * psStream )
{
	DownloadNode *psNode = new DownloadNode;

	psNode->m_nStartTime = get_system_time();
	psNode->m_cURL = psStream->m_cURL;
	psNode->m_cPreferredName = psStream->m_cPreferredName;
	psNode->m_nContentSize = psStream->m_nContentSize;

	try
	{
		psNode->m_pcTmpFile = new os::TempFile( "ab-", "/tmp" );
	} catch( ... )
	{
		printf( "Error: BrowserWindow::BeginDownload() failed to create temporary file\n" );
		return ( false );
	}

	Message cOkMsg( ID_START_DOWNLOAD );
	Message cCancelMsg( ID_CANCEL_DOWNLOAD );

	cOkMsg.AddPointer( "node", psNode );
	cCancelMsg.AddPointer( "node", psNode );

	psNode->m_psDownloadStream = psStream;
	psNode->m_pcRequester = new DownloadRequester( Rect( 250, 200, 599, 300 ), "File Download", psStream->m_cURL, psStream->m_cPreferredName, psStream->m_cMimeType, psStream->m_nContentSize, Messenger( this ), cOkMsg, cCancelMsg );
	psNode->m_pcRequester->Show();
	psNode->m_pcRequester->MakeFocus();
	psStream->m_pUserData = psNode;
	return ( true );
}

bool BrowserWindow::HandleDownloadData( KParts::DownloadStream * psStream, const void *pData, int nLen )
{
	DownloadNode *psNode = static_cast < DownloadNode * >( psStream->m_pUserData );

	if ( psNode == NULL || psNode->m_bCanceled || ( psNode->m_pcTmpFile == NULL && psNode->m_pcFile == NULL ) )
	{
		return ( false );
	}
	int nBytesWritten;

	if ( psNode->m_pcFile != NULL )
	{
		nBytesWritten = psNode->m_pcFile->Write( pData, nLen );
	}
	else
	{
		nBytesWritten = psNode->m_pcTmpFile->Write( pData, nLen );
	}
	if ( nBytesWritten != nLen )
	{
		( new os::Alert( "Error: Failed to create file!", os::String ().Format( "Failed to write %df bytes to\n%s\n\nError: %s\n", psNode->m_cPath.c_str(  ), strerror( errno ) ), 0, "Sorry!", NULL ) )->Go( NULL );

		psNode->m_bCanceled = true;

		if ( psNode->m_pcProgressDlg != NULL )
		{
			psNode->m_pcProgressDlg->Terminate();
			psNode->m_pcProgressDlg = NULL;
		}
		if ( psNode->m_pcRequester == NULL )
		{
			delete psNode->m_pcTmpFile;
			delete psNode->m_pcFile;

			if ( psNode->m_cPath.empty() == false )
			{
				unlink( psNode->m_cPath.c_str() );
			}
			delete psNode;
		}
		else
		{
			psNode->m_bDone = true;
		}

	}
	else
	{
		psNode->m_nBytesReceived += nLen;
		if ( psNode->m_pcProgressDlg != NULL )
		{
			psNode->m_pcProgressDlg->UpdateProgress( psNode->m_nBytesReceived );
		}
	}
	return ( true );
}

void BrowserWindow::DownloadFinished( KParts::DownloadStream * psStream, int /* nError */  )
{
	DownloadNode *psNode = static_cast < DownloadNode * >( psStream->m_pUserData );

	if ( psNode == NULL || psNode->m_bCanceled || ( psNode->m_pcTmpFile == NULL && psNode->m_pcFile == NULL ) )
	{
		return;
	}

	if ( psNode->m_pcProgressDlg != NULL )
	{
		psNode->m_pcProgressDlg->Terminate();
		psNode->m_pcProgressDlg = NULL;
	}
	if ( psNode->m_pcRequester == NULL )
	{
		delete psNode->m_pcTmpFile;
		delete psNode->m_pcFile;
		delete psNode;
	}
	else
	{
		psNode->m_bDone = true;
	}
}

void BrowserWindow::HandleMessage( Message * pcMsg )
{
	switch ( pcMsg->GetCode() )
	{
	case ID_TAB_CHANGED:
		{
			SwitchTab();
			break;
		}
	case ID_NEW_TAB:
		{
			std::string cURL;
			
			if (pcMsg->FindString( "url", &cURL )==0)
				CreateNewTab( cURL, KParts::URLArgs() );
			else		
				CreateNewTab();
			break;
		}
	case ID_DELETE_TAB:
		{
			DeleteTab();
			break;
		}
	case ID_URL_CHANGED:
		{
			uint32 nEvents = 0;

			pcMsg->FindInt( "events", &nEvents );
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			if ( nEvents & os::TextView::EI_ESC_PRESSED )
			{
				m_pcURLView->SetCurrentString(hview->GetCurrentURL().c_str());
				hview->GetPart()->view(  )->MakeFocus(  );
				break;
			}
			if ( nEvents & os::TextView::EI_ENTER_PRESSED )
			{
				std::string cURL = m_pcURLView->GetCurrentString();

				for ( uint i = 0; i <= cURL.size(); ++i )
				{
					if ( i == cURL.size() || isalnum( cURL[i] ) == false )
					{
						if ( i == cURL.size() || cURL[i] != ':' )
						{
							std::string cTmp = cURL;
							cURL = "http://";
							for ( uint j = 0; j < cTmp.size(); ++j )
							{
								if ( cTmp[j] != '/' )
								{
									cURL.insert( cURL.end(), cTmp.begin(  ) + j, cTmp.end(  ) );
									break;
								}
							}
						}
						break;
					}
				}

				hview->GetPart()->view(  )->MakeFocus(  );
				OpenURLRequest( cURL, KParts::URLArgs() );

			}
			break;
		}
		
	case ID_GO_ACTIVATED:
	{
		uint i = m_pcTabs->GetSelection();
		HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
		std::string cURL = m_pcURLView->GetCurrentString();
		hview->GetPart()->view(  )->MakeFocus(  );
		OpenURLRequest( cURL, KParts::URLArgs() );
		break;
	}
	
	case ID_STOP:
	{
		uint i = m_pcTabs->GetSelection();
		HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
		hview->GetPart()->closeURL();		
		break;
	}
	
	
	case ID_ACTIVATE_URLEDIT:
		m_pcURLView->MakeFocus( false );
		m_pcURLView->GetTextView()->SetCursor( -1, 0 );
		m_pcURLView->SelectAll();
		m_pcURLView->MakeFocus();
		SetFocusChild(m_pcURLView);
		break;
		
		
		
	case ID_FILE_OPEN_LOCATION:
	{
		if (m_pcInputBox == NULL)
		{
			m_pcInputBox = new os::InputBox(new os::Messenger(this),"Open Site","Enter a website here:");
			m_pcInputBox->CenterInWindow(this);
			m_pcInputBox->Show();
			m_pcInputBox->MakeFocus();
		}
		else
			m_pcInputBox->MakeFocus();
		break;
	}
	
	case M_BOX_REQUESTED:
	{
		os::String cPath = m_pcInputBox->GetValue().AsString();
		if (cPath[0] == '/')
		{
			cPath = "file:" + cPath;
		}	
		else if (cPath.find("http") == std::string::npos)
		{
			cPath = "http://" + cPath;
		}
		
		KParts::URLArgs cArgs;
		OpenURLRequest(cPath,cArgs);
		m_pcInputBox = NULL;
		break;		
	}
		
	case ID_FILE_OPEN_FILE:
	{
		if (m_pcRequester == NULL)
		{
			m_pcRequester = new os::FileRequester(os::FileRequester::LOAD_REQ,new os::Messenger(this));
			m_pcRequester->CenterInWindow(this);
			m_pcRequester->Show();
			m_pcRequester->MakeFocus();
		}
		
		else
			m_pcRequester->MakeFocus();
		break;
	}
	
	case M_LOAD_REQUESTED:
	{
		os::String cPath;
		
		if (pcMsg->FindString("file/path",&cPath) == 0)
		{
			KParts::URLArgs cArgs;
			cPath = "file:" + cPath;		
			OpenURLRequest(cPath,cArgs);
		}
		m_pcRequester = NULL;
		break;
	}
	
	case ID_RELOAD:
		{
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			KParts::URLArgs cArgs;
			cArgs.reload = true;
			cArgs.xOffset = hview->GetPart()->browserExtension(  )->xOffset(  );
			cArgs.yOffset = hview->GetPart()->browserExtension(  )->yOffset(  );
			OpenURL( m_pcURLView->GetCurrentString(), cArgs );
			break;
		}
		
	case ID_VIEW_SOURCE:
	{
		
		os::TempFile* pcTmpFile;
		try
		{
			pcTmpFile = new os::TempFile( "ab-", "/tmp" );
		} catch( ... )
		{
		}
		
		uint i = m_pcTabs->GetSelection();
		HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
		
		QString cEscapedHTML = hview->GetPart()->documentSource();
		
		if (m_pcSettings->GetOpenSourceInEditorOption() && m_pcSettings->GetSourceEditor() != "")
		{
			pcTmpFile->Write(os::String(cEscapedHTML).c_str(),os::String(cEscapedHTML).size());
			pcTmpFile->Flush();
			if (fork() == 0)
			{
				execlp(m_pcSettings->GetSourceEditor().c_str(),m_pcSettings->GetSourceEditor().c_str(),pcTmpFile->GetPath().c_str(),NULL);
			}
			break;
		}
		else
		{
			os::String cString = os::String().Format("<html><head><title>Viewing Source for %s\n</title></head><body><pre><code>\n",m_pcURLView->GetCurrentString().c_str());
			KParts::URLArgs cArgs;			
		
			/*escape the html code*/
			cEscapedHTML.replace('<',"&lt;");
			cEscapedHTML.replace('>',"&gt;");
		
			cString += cEscapedHTML + "</code></pre></body>";
			pcTmpFile->Write(cString.c_str(),cString.size());
			pcTmpFile->Flush();
		
		
			CreateNewTab("file:"+ pcTmpFile->GetPath(),cArgs);
		}
		break;
	}
	
	case ID_NEW_WINDOW:
		{
			UpdateHistory();
			BrowserWindow *pcNewWindow = new BrowserWindow( GetFrame() + os::Point( 30.0f, 30.0f ), false );

			pcNewWindow->GoHistory( 0, false );
			pcNewWindow->Show( true );
			pcNewWindow->MakeFocus( true );
			break;
		}
	case ID_OPEN_TAB:
		{
			std::string cURL;
			pcMsg->FindString( "url", &cURL );
			CreateNewTab( cURL, KParts::URLArgs() );
			break;
		}
	case ID_OPEN_LINK:
		{
			std::string cURL;
			pcMsg->FindString( "url", &cURL );
			BrowserWindow *pcNewWindow = new BrowserWindow( GetFrame() + os::Point( 30.0f, 30.0f ), false );

			pcNewWindow->AddHistoryEntry();
			pcNewWindow->OpenURL( cURL, KParts::URLArgs() );
			pcNewWindow->Show( true );
			pcNewWindow->MakeFocus( true );
			break;
		}
	case ID_SAVE_LINK:
		{
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			std::string cURL;
			if (pcMsg->FindString( "url", &cURL )!=0)
				cURL = m_pcURLView->GetCurrentString();

			hview->GetPart()->browserExtension(  )->SaveURL( cURL, "Save link as..." );
			break;
		}
	case ID_COPY_LINK_LOCATION:
		{
			std::string cURL;
			pcMsg->FindString( "url", &cURL );

			os::Clipboard cClipboard;
			cClipboard.Lock();
			cClipboard.Clear();
			os::Message * pcData = cClipboard.GetData();
			pcData->AddString( "text/plain", cURL );
			cClipboard.Commit();
			cClipboard.Unlock();
			break;
		}
	case ID_PASTE:
		{
			GetURLView()->Delete(  );
			GetURLView()->Paste( false );
			break;
		}
		
	case ID_COPY:
		{
			bool bCopyURL = false;
			if (m_pcURLView->HasFocus())
			{
				bCopyURL = true;
			}
						
			else if (m_cSelectedText.empty())
			{
				break;
			}
			
			
			os::Clipboard cClipboard;
			cClipboard.Lock();
			cClipboard.Clear();
			os::Message * pcData = cClipboard.GetData();
			
			if (bCopyURL)
				pcData->AddString("text/plain",m_pcURLView->GetTextView()->GetBuffer()[0]);
			else
				pcData->AddString( "text/plain", m_cSelectedText );
			cClipboard.Commit();
			cClipboard.Unlock();
			break;
		}
		
	case ID_SELECT_ALL:
		{
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			hview->GetPart()->selectAll();
			break;
		}
			
	
	case ID_VIEW_SHOW_BOOKMARK:
	{
		void* menu;
		pcMsg->FindPointer("source",&menu);
		os::CheckMenu* temp = static_cast<os::CheckMenu*>(menu);
		m_bBookmarkToolBar = temp->IsChecked();
		Layout();
		break;
	}
	
	case ID_VIEW_SHOW_MAIN_TOOLBAR:
	{
		void* menu;
		pcMsg->FindPointer("source",&menu);
		os::CheckMenu* temp = static_cast<os::CheckMenu*>(menu);
		m_bMainToolBar = temp->IsChecked();
		Layout();
		break;		
	}
	
	case ID_FIND:
	{
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			hview->GetPart()->findTextBegin(  );
			m_bFindPanel = true;
			Layout();
		break;
	}
	
	case FindPanel::FIND_NEXT:
		{
			bool bFromTop = true;
			pcMsg->FindString( "find", &m_cLastSearchStr );

			KHTMLView *pcActiveView = dynamic_cast < KHTMLView * >( GetFocusChild() );
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );
			KHTMLPart *pcPart = hview->GetPart();

			if ( pcActiveView != NULL )
			{
				pcPart = pcActiveView->part();
			}
			if ( bFromTop )
			{
				pcPart->findTextBegin();
			}
			pcPart->findTextNext( m_cLastSearchStr.c_str(), true, m_bMatchCase, false );
			break;
		}
		
	
	case FindPanel::FIND_PREV:
		{
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			if ( m_cLastSearchStr.empty() == false )
			{
				KHTMLView *pcActiveView = dynamic_cast < KHTMLView * >( GetFocusChild() );

				KHTMLPart *pcPart = hview->GetPart();

				if ( pcActiveView != NULL )
				{
					pcPart = pcActiveView->part();
				}
				pcPart->findTextNext( m_cLastSearchStr.c_str(), true, m_bMatchCase, true );
			}
			break;
		}
	case FindPanel::CLOSE:
		m_bFindPanel = false;
		Layout();
		break;
		
	case FindPanel::MATCH_CASE:
		pcMsg->FindBool("match",&m_bMatchCase);
		break;
		
	case ID_GO_HOME:
		UpdateHistory();
		AddHistoryEntry();
		OpenURL(m_cDefaultPath, KParts::URLArgs() );
		UpdateHistory();
		break;
	case ID_PREV_URL:
		GoHistory( -1, false );
		break;
	case ID_NEXT_URL:
		GoHistory( 1, false );
		break;
	case ID_START_DOWNLOAD:
		{
			DownloadNode *psNode = NULL;

			std::string cPath;
			pcMsg->FindPointer( "node", ( void ** )&psNode );
			pcMsg->FindString( "path", &cPath );


			if ( psNode == NULL )
			{
				break;	// Should never happen
			}
			psNode->m_cPath = cPath;
			const std::string cDstDir = os::Path( cPath.c_str() ).GetDir(  ).GetPath(  ).c_str(  );

			try
			{
				os::FSNode cDirNode( cDstDir );
				if ( cDirNode.GetDev() == psNode->m_pcTmpFile->GetDev(  ) )
				{
					rename( psNode->m_pcTmpFile->GetPath().c_str(  ), cPath.c_str(  ) );
					psNode->m_pcTmpFile->Detatch();	// Make sure the destructor don't do anything silly.
				}
				else
				{
					psNode->m_pcFile = new os::File( cPath, O_WRONLY | O_CREAT );
					psNode->m_pcTmpFile->Seek( 0, SEEK_SET );
					char anBuffer[32 * 1024];

					for ( ;; )
					{
						int nLen = psNode->m_pcTmpFile->Read( anBuffer, sizeof( anBuffer ) );

						if ( nLen < 0 )
						{
							throw os::errno_exception( "Failed to copy temporary file" );
						}
						if ( psNode->m_pcFile->Write( anBuffer, nLen ) != nLen )
						{
							throw os::errno_exception( "Failed to copy temporary file" );
						}
						if ( nLen < int ( sizeof( anBuffer ) ) )
						{
							break;
						}
					}
					delete psNode->m_pcTmpFile;

					psNode->m_pcTmpFile = NULL;
				}
			}
			catch( ... )
			{
				( new os::Alert( "Error: Failed to create file!", os::String ().Format( "Failed to create '%s'", cPath.c_str(  ) ), 0, "Sorry!", NULL ) )->Go( NULL );
			}

			psNode->m_pcRequester = NULL;

			if ( psNode->m_bDone == false )
			{
				Message cCancelMsg( ID_CANCEL_DOWNLOAD );

				cCancelMsg.AddPointer( "node", psNode );

				psNode->m_pcProgressDlg = new DownloadProgress( os::Rect( 100, 100, 449, 349 ), os::String ().Format( "Download: %s", cPath.c_str(  ) ), psNode->m_cURL, cPath, psNode->m_nContentSize, psNode->m_nStartTime, os::Messenger( this ), cCancelMsg );

				psNode->m_pcProgressDlg->Show();
			}
			else
			{
				delete psNode->m_pcTmpFile;
				delete psNode->m_pcFile;
				delete psNode;
			}
			break;
		}
	case ID_CANCEL_DOWNLOAD:
		{
			DownloadNode *psNode = NULL;

			pcMsg->FindPointer( "node", ( void ** )&psNode );

			if ( psNode == NULL )
			{
				break;	// Should never happen
			}
			if ( psNode->m_pcProgressDlg != NULL )
			{
				psNode->m_pcProgressDlg->Terminate();
				psNode->m_pcProgressDlg = NULL;
			}
			if ( psNode->m_cPath.empty() == false )
			{
				unlink( psNode->m_cPath.c_str() );
			}
			if ( psNode->m_bDone == false )
			{
				psNode->m_pcRequester = NULL;
				psNode->m_bCanceled = true;
				delete psNode->m_pcTmpFile;

				psNode->m_pcTmpFile = NULL;
				delete psNode->m_pcFile;

				psNode->m_pcFile = NULL;
			}
			else
			{
				delete psNode->m_pcTmpFile;
				delete psNode->m_pcFile;
				delete psNode;
			}
			break;
		}
	case ID_ABOUT:
		{
			os::Alert * pcAbout = new os::Alert( "About ABrowse", "ABrowse 0.4 Alpha3\nA KHTML based webbrowser\nWritten by Kurt Skauen, 2001\nWritten by Arno Klenke, 2004" "\n\nABrowse is released under the Gnu Public Licencse (GPL)\nPlease see the file COPYING, distributed with ABrowse, for\nmore information\n", os::Alert::ALERT_INFO, 0x00, "O.K", NULL );
			pcAbout->Go( new os::Invoker() );
			break;
		}
	case ID_ADD_BOOKMARK:
		{
			os::String zPath;
			bool bDummy;
			
			if( pcMsg->FindBool( "slave", &bDummy ) == 0 )
			{
				/* Update menu */
				m_pcBookmarkMenu->Reload();
				m_pcBookmarkMenu->SetTargetForItems( this );
				
				m_pcBookmarkToolBar->Reload();
				break;
			}
			
			if( pcMsg->FindString( "path", &zPath ) != 0 )
				break;
			
			
			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			if ( m_pcTabs->GetTabTitle( i ) == "Loading..." )
				break;
			os::String zFile = m_pcTabs->GetTabTitle( i );

			for ( i = 0; i < zFile.Length(); i++ )
				if ( zFile[i] == '/' )
					zFile[i] = '_';
			zFile = zPath + "/" + zFile;

			try
			{
				os::String zType = "text/x-bookmark";
				os::File cFile( zFile, O_RDWR | O_CREAT | O_TRUNC );
				cFile.Write( hview->GetCurrentURL().c_str(  ), hview->GetCurrentURL(  ).Length(  ) );
				cFile.WriteAttr( "os::MimeType", O_TRUNC, ATTR_TYPE_STRING, zType.c_str(), 0, zType.Length() );
				cFile.Flush();
			} catch( ... )
			{
			}
			/* Update menu */
			m_pcBookmarkMenu->Reload();
			m_pcBookmarkMenu->SetTargetForItems( this );
			
			m_pcBookmarkToolBar->Reload();
			
			/* Tell the other windows */
			
			for( uint i = 0; i < s_cWindows.size(); i++ )
			{
				if( s_cWindows[i] != this )
				{
					os::Message cMsg( *pcMsg );
					cMsg.AddBool( "slave", true );
					s_cWindows[i]->PostMessage( &cMsg, s_cWindows[i] );
				}
			}
			
			break;
		}
	case ID_MANAGE_BOOKMARKS:
		{
			os::String zPath;
			if( pcMsg->FindString( "path", &zPath ) != 0 )
				break;
			/* Open a filebrowser window */
			
			if ( fork() == 0 )
			{
				set_thread_priority( -1, 0 );
				execlp( "/system/bin/FileBrowser", "/system/bin/FileBrowser", zPath.c_str(), NULL );
			}
			break;
		}
		
		
	case ID_BOOKMARK_TOOLBAR_MANAGE:
	{		
			if ( fork() == 0 )
			{
				set_thread_priority( -1, 0 );
				execlp( "/system/bin/FileBrowser", "/system/bin/FileBrowser", GetBookMarkToolBarPath().c_str(), NULL );
			}
			break;
	}
	
	case ID_BOOKMARK_TOOLBAR_ADD:
		{
			os::String zPath;
			bool bDummy;
			
			if( pcMsg->FindBool( "slave", &bDummy ) == 0 )
			{
				m_pcBookmarkMenu->Reload();
				m_pcBookmarkMenu->SetTargetForItems( this );
			
				m_pcBookmarkToolBar->Reload();
				break;
			}

			uint i = m_pcTabs->GetSelection();
			HTMLView *hview = ( HTMLView * ) m_pcTabs->GetTabView( i );

			if ( m_pcTabs->GetTabTitle( i ) == "Loading..." )
				break;
			os::String zFile = m_pcTabs->GetTabTitle( i );

			for ( i = 0; i < zFile.Length(); i++ )
				if ( zFile[i] == '/' )
					zFile[i] = '_';
			zFile = GetBookMarkToolBarPath() + "/" + zFile;

			try
			{
				os::String zType = "text/x-bookmark";
				os::File cFile( zFile, O_RDWR | O_CREAT | O_TRUNC );
				cFile.Write( hview->GetCurrentURL().c_str(  ), hview->GetCurrentURL(  ).Length(  ) );
				cFile.WriteAttr( "os::MimeType", O_TRUNC, ATTR_TYPE_STRING, zType.c_str(), 0, zType.Length() );
				cFile.Flush();
			} catch( ... )
			{
			}
			/* Update menu */
			m_pcBookmarkMenu->Reload();
			m_pcBookmarkMenu->SetTargetForItems( this );
			m_pcBookmarkToolBar->Reload();
			
			/* Tell the other windows */
			
			for( uint i = 0; i < s_cWindows.size(); i++ )
			{
				if( s_cWindows[i] != this )
				{
					os::Message cMsg( *pcMsg );
					cMsg.AddBool( "slave", true );
					s_cWindows[i]->PostMessage( &cMsg, s_cWindows[i] );
				}
			}
			
			break;
		}	
		
	case ID_TOOLS_READ_MAIL:
	{
		/*TODO: Make it use default mail application via registrar/events*/
		if ( fork() == 0 )
		{
			set_thread_priority( -1, 0 );
			execlp( "/Applications/Whisper/Whisper", "/Applications/Whisper/Whisper",NULL );
		}
		break;
	}
	case ID_GO_BOOKMARK:
		{
			os::String zPath;
			if( pcMsg->FindString( "file/path", &zPath ) != 0 )
				break;
			char zBuffer[PATH_MAX];
			memset( zBuffer, 0, sizeof( zBuffer ) );
			try
			{
				os::File cFile( zPath );
				if( cFile.Read( zBuffer, PATH_MAX ) <= 0 )
					break;
				OpenURLRequest( zBuffer, KParts::URLArgs() );
			} catch( ... ) {
			}
			break;
		}
	case os::M_NODE_MONITOR:
		{
			break;
		}
	case ID_POPUP_CLOSED:
		{
			break;
		}
	case M_SETTINGS_REQUESTED:
	{
		if (m_pcSettingsDialog->IsVisible())
			m_pcSettingsDialog->MakeFocus();
		else
		{
			m_pcSettingsDialog->CenterInScreen();
			m_pcSettingsDialog->Show();
			m_pcSettingsDialog->MakeFocus();
		}
		break;
	}
	case M_SETTINGS_CLOSED:
	{
		bool save;
		m_pcSettingsDialog->Show(false);
		
		if (pcMsg->FindBool("save",&save) == 0)
		{
			m_cDefaultPath = m_pcSettings->GetDefaultPath().str();
		}
		break;
	}
	
	default:
		Window::HandleMessage( pcMsg );
		break;
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool BrowserWindow::OkToQuit( void )
{
	if ( s_cWindows.size() == 1 )
	{
		m_pcSettings->SetFrame(GetFrame());
		m_pcSettings->SaveSettings();
		
		Application::GetInstance()->PostMessage( M_TERMINATE );
	}
	return ( true );
}













































