#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream>

#include <atheos/time.h>

#include <util/application.h>
#include <util/message.h>
#include <util/messenger.h>
#include <util/clipboard.h>
#include <util/resources.h>
#include <gui/window.h>
#include <gui/view.h>
#include <gui/frameview.h>
#include <gui/button.h>
#include <gui/textview.h>
#include <gui/tabview.h>


//all khtml files
#include <khtml_part.h>
#include <khtmlview.h>
#include <khtml_factory.h>
#include <misc/loader.h>
#include <kapplication.h>
#include <klibloader.h>
#include <ecma/kjs_proxy.h>
#include <kiconloader.h>
#include <css/cssstyleselector.h>
#include <rendering/render_style.h>

#include <string>
#include <list>

#include <gui/menu.h>
#include <gui/requesters.h>
#include <storage/path.h>
#include <storage/file.h>
#include <storage/directory.h>
#include <storage/tempfile.h>
#include <storage/registrar.h>
#include <util/event.h>
#include <gui/toolbar.h>
#include <gui/statusbar.h>
#include <gui/progressbar.h>
#include <gui/separator.h>
#include <gui/checkmenu.h>
#include <gui/inputbox.h>
#include <gui/filerequester.h>

#include <qmutex.h>


#include "downloaddialog.h"
#include "findpanel.h"
#include "addresspanel.h"
#include "utils.h"
#include "bookmarks.h"
#include "application.h"
#include "bookmarkstoolbar.h"
#include "settings.h"
#include "settingsdialog.h"

#include "HTMLView.h"
#include "../kio/http/kcookiejar/kcookiejar.h"

extern KCookieJar g_cCookieJar;
static KApplication *g_pcKApp;

using namespace os;

struct DownloadNode
{
	DownloadNode()
	{
		m_psDownloadStream = NULL;
		m_pcRequester = NULL;
		m_pcProgressDlg = NULL;
		m_nContentSize = -1;
		m_pcTmpFile = NULL;
		m_pcFile = NULL;
		m_nBytesReceived = 0;
		m_bDone = false;
		m_bCanceled = false;
	}

	KParts::DownloadStream * m_psDownloadStream;
	DownloadRequester *m_pcRequester;
	DownloadProgress *m_pcProgressDlg;

	std::string m_cURL;
	std::string m_cPreferredName;
	off_t m_nContentSize;
	bigtime_t m_nStartTime;

	std::string m_cPath;
	os::TempFile * m_pcTmpFile;
	os::File * m_pcFile;
	int m_nBytesReceived;
	bool m_bDone;
	bool m_bCanceled;
};

class BrowserWindow:public Window, public KParts::BrowserCallback
	{
	public:
		BrowserWindow( const Rect & cFrame,bool bLoadPos );
		 ~BrowserWindow();
		
		static os::Point GetNewWinPosition();
		void OpenURL( const std::string & cURL, const KParts::URLArgs & cArgs );
		void OpenFromArgs(const os::String&);
		void OpenDefault();
		void EnableActions();

		//tabview methods
		void CreateNewTab();
		KParts::ReadOnlyPart * CreateNewTab( const std::string & cURL, const KParts::URLArgs & cArgs );
		void SwitchTab();
		void DeleteTab();
		
		virtual void HandleMessage( Message * pcMsg );

		// From Window:
		bool OkToQuit();

		// From BrowserCallback:
		virtual void EnableAction( const char * name , bool enabled   )
		{
		}
		virtual void OpenURLRequest( const std::string & url, const KParts::URLArgs & args );
		virtual void OpenURLNotify();
		virtual void Completed();
		virtual void SetLocationBarURL( const std::string & url );
		virtual void CreateNewWindow( const std::string & url, const KParts::URLArgs & args );
		virtual KParts::ReadOnlyPart * CreateNewWindow( const std::string & url, const KParts::URLArgs & args, const KParts::WindowArgs & windowArgs );
		virtual void PopupMenu( KParts::BrowserExtension * pcSource, const std::string & cURL );
		virtual void LoadingProgress( int percent );
		virtual void SpeedProgress( int bytesPerSecond );
		virtual void InfoMessage( const std::string & cString );
		virtual void SetWindowCaption( KParts::ReadOnlyPart * pcPart, const std::string & caption );
		virtual void SetStatusBarText( const std::string & text );
		virtual void SelectionInfo( const std::string & text );

		virtual bool BeginDownload( KParts::DownloadStream * psStream );
		virtual bool HandleDownloadData( KParts::DownloadStream * psStream, const void *pData, int nLen );
		virtual void DownloadFinished( KParts::DownloadStream * psStream, int nError );

		// New members:
		AddressPanel *GetURLView();
		KHTMLPart *GetPart();

	private:
		void SetupFindPanel();
		void SetupMenus();
		void SetupTabBar();
		void SetupToolBar();
		void SetupStatusBar();
		void LoadSettings();
		void SetupBookmarksToolBar();
		
		void Init(bool);
		void Layout();
		void AddFrame();
		void RegisterEvents();
		
		void UpdateHistory();
		void AddHistoryEntry();
		void GoHistory( int nStep, bool bButtonsOnly );
		
		void EnableButton(const os::String& cName,bool bEnable);
		os::ImageButton* AddButton(const os::String& cIcon,const os::String& cName,os::Message* pcMessage);
		
		os::Menu* GetNewMenu();
		os::Menu* GetCloseMenu();
		os::Menu* GetOpenMenu();
		os::Menu* GetPrintMenu();
		os::Menu* GetToolBarMenu();
		os::Menu* GetTextSizeMenu();
		
		static std::vector<BrowserWindow*>	s_cWindows;
		
		std::vector<os::ImageButton*>		m_sToolBarButtons;
		std::vector<os::Menu*>				m_sMenus;
		
		std::string							m_cSelectedText;
		std::string							m_cLastSearchStr;
		std::string							m_cDefaultPath;
		bool								m_bSearchCaseSensitive;
		
		os::ToolBar*						m_pcToolBar;
		StatusBar*							m_pcStatusBar;
		ProgressBar*						m_pcProgressBar;
		AddressPanel*						m_pcURLView;

		os::Menu*							m_pcMenuBar;
		os::Menu*							m_pcPopupMenu;
		os::Menu*							m_pcTabMenu;
		BookmarksMenu*						m_pcBookmarkMenu;
		
	
		BookmarksToolBar*					m_pcBookmarkToolBar;
		
		os::TabView*						m_pcTabs;
		FindPanel*							m_pcFindPanel;

		os::Rect							m_cFrame;
		LayoutView*							m_pcFrame;
		
		
		os::FileRequester*					m_pcRequester;
		os::InputBox*						m_pcInputBox;
		
		bool								m_bSaveSize;
		bool								m_bFindPanel;
		bool								m_bBookmarkToolBar;
		bool								m_bMainToolBar;
		bool								m_bStatusBar;
		bool								m_bMatchCase;
		


		ABrowse_Settings*					m_pcSettings;
		os::Event*							m_pcOpenInNewTabEvent;
		os::Event*							m_pcOpenInNewWindowEvent;

		SettingsDialog*						m_pcSettingsDialog;		
		
		friend class 						BrowserApp;
	};


#endif


























