
#include "settingsdialog.h"
#include "settings.h"
#include "utils.h"
#include "messages.h"


#include <util/settings.h>
#include <util/shortcutkey.h>
#include <gui/checkbox.h>

#include <stdio.h>
#include <stdarg.h>

using namespace os;

class GeneralSettings : public SettingsNode
{
public:
	GeneralSettings(ABrowse_Settings* settings) : SettingsNode("General")
	{
		m_pcSettings = settings;
	}
	
	void LayoutSystem()
	{
		SetHAlignment(os::ALIGN_LEFT);
		SetVAlignment(os::ALIGN_TOP);
		os::HLayoutNode* pcNode = new HLayoutNode("path");
		pcNode->SetHAlignment(os::ALIGN_LEFT);
		pcNode->SetVAlignment(os::ALIGN_TOP);		
		pcNode->AddChild(m_pcHomeStringView = new os::StringView(os::Rect(),"home_string","Home:"));
		pcNode->AddChild(new HLayoutSpacer("",5,5));
		pcNode->AddChild(m_pcHomeTextView = new os::DropdownMenu(os::Rect(),"home_text"));
		m_pcHomeTextView->SetMinPreferredSize(20);
		m_pcHomeTextView->SetMaxPreferredSize(20);
		AddChild(pcNode);
	}
	
	void LoadSettings()
	{
		m_pcHomeTextView->SetCurrentString(m_pcSettings->GetDefaultPath());
	}
	
	void SaveSettings()
	{
		m_pcSettings->SetDefaultPath(m_pcHomeTextView->GetCurrentString());
	}
	
	void SetTabs()
	{
	}
	
	os::BitmapImage* GetSettingsIcon()
	{
		return LoadImageFromResource("general.png");
	}
	
	os::String GetSettingsName()
	{
		return "General";
	}
private:
	ABrowse_Settings* m_pcSettings;
	
	os::DropdownMenu* m_pcHomeTextView;
	os::StringView* m_pcHomeStringView;
};

class DefaultAppSettings : public SettingsNode
{
public:
	DefaultAppSettings(ABrowse_Settings* settings, os::Looper* pcHandler) : SettingsNode("DefaultApp")
	{
		m_pcHandler = pcHandler;
		m_pcSettings = settings;
	}
		
	void LayoutSystem()
	{
		SetHAlignment(os::ALIGN_LEFT);
		SetVAlignment(os::ALIGN_TOP);
		
		os::VLayoutNode* pcNode = new VLayoutNode("path");
		pcNode->SetHAlignment(os::ALIGN_LEFT);
		pcNode->SetVAlignment(os::ALIGN_TOP);
		
		pcNode->AddChild(m_pcOpenSourceCheck = new os::CheckBox(os::Rect(),"source_check","Open source with external applicaton",new os::Message( M_SETTINGS_SOURCE_CHANGED)));		
		m_pcOpenSourceCheck->SetTarget(m_pcHandler);
		pcNode->AddChild(new VLayoutSpacer("",10,10));
		pcNode->AddChild(m_pcSourceStringView = new os::StringView(os::Rect(),"source_string","Editor:"));
		pcNode->AddChild(new HLayoutSpacer("",5,5));
		pcNode->AddChild(m_pcSourceTextView = new os::DropdownMenu(os::Rect(),"source_text"));
		m_pcSourceTextView->SetMinPreferredSize(20);
		m_pcSourceTextView->SetMaxPreferredSize(20);
    	pcNode->ExtendMinSize(pcNode->GetPreferredSize(false));
    	pcNode->LimitMaxSize(pcNode->GetPreferredSize(false));
    	pcNode->ExtendMaxSize(pcNode->GetPreferredSize(false));
		AddChild(pcNode);			
	}
		
	void LoadSettings()
	{
		m_pcOpenSourceCheck->SetValue(m_pcSettings->GetOpenSourceInEditorOption());
		
		if (m_pcSettings->GetOpenSourceInEditorOption())
			m_pcSourceTextView->SetCurrentString(m_pcSettings->GetSourceEditor());
		else
			m_pcSourceTextView->SetCurrentString("Default");
	}
	
	void SaveSettings()
	{
		m_pcSettings->SetOpenSourceInEditorOption(m_pcOpenSourceCheck->GetValue().AsBool());
		
		
		if (m_pcOpenSourceCheck->GetValue().AsBool())
			m_pcSettings->SetSourceEditor(m_pcSourceTextView->GetCurrentString());
		else
			m_pcSettings->SetSourceEditor("Default");
	}
		
	void SetTabs()
	{
	}
		
	os::BitmapImage* GetSettingsIcon()
	{
		return LoadImageFromResource("app.png");
	}
		
	os::String GetSettingsName()
	{
		return "Apps";
	}
	
	void SourceChanged()
	{
		if (m_pcOpenSourceCheck->GetValue().AsBool())
			m_pcSourceTextView->SetEnable(true);
		else
		{
			m_pcSourceTextView->SetEnable(false);
			m_pcSourceTextView->SetCurrentString("Default");
		}
	}
private:
	os::Looper* m_pcHandler;
	ABrowse_Settings* m_pcSettings;
	
	os::DropdownMenu* m_pcSourceTextView;
	os::StringView* m_pcSourceStringView;
	os::CheckBox* m_pcOpenSourceCheck;
};


class SettingsIconView : public os::IconView
{
public:
		SettingsIconView() : os::IconView(os::Rect(),"",os::CF_FOLLOW_NONE)
		{
		}
		
		os::Point GetPreferredSize(bool) const
		{
			return os::Point(60,200);
		}
};

SettingsDialog::SettingsDialog(os::Messenger* pcTarget, ABrowse_Settings* settings) : Window(os::Rect(0,0,500,200),"side_settings_window","Global Settings...",os::WND_NOT_RESIZABLE)
{
	m_pcView = new os::LayoutView(GetBounds(),"settings_view",NULL,os::CF_FOLLOW_ALL);
	
	m_pcTarget = pcTarget;
	m_pcSettings = settings;

	m_cSettingNodes.push_back(new GeneralSettings(m_pcSettings));
	m_cSettingNodes.push_back(new DefaultAppSettings(m_pcSettings,this));
	
	for (uint i=0; i<m_cSettingNodes.size(); i++)
	{
		m_cSettingNodes[i]->LayoutSystem();
		m_cSettingNodes[i]->LoadSettings();
	}
	os::VLayoutNode* m_pcRoot = new os::VLayoutNode("root");
	m_pcRoot->SetVAlignment(os::ALIGN_RIGHT);
	m_pcRoot->SetHAlignment(os::ALIGN_RIGHT);
	
	m_pcHLayout = new os::HLayoutNode("major");
	m_pcHLayout->SetBorders(os::Rect(5,5,5,5));	
	m_pcHLayout->SetVAlignment(os::ALIGN_LEFT);
	m_pcHLayout->SetHAlignment(os::ALIGN_LEFT);
	
	m_pcIconView = new SettingsIconView();
	m_pcIconView->SetAutoSort(false);
	
	for (uint i=0; i<m_cSettingNodes.size(); i++)
		m_pcIconView->AddIconString( m_pcIconView->AddIcon(m_cSettingNodes[i]->GetSettingsIcon(),new os::IconData()),m_cSettingNodes[i]->GetSettingsName());
	
	m_pcIconView->SetView(os::IconView::VIEW_ICONS);
	m_pcIconView->Layout();
	
	m_pcIconView->SetSelChangeMsg(new os::Message(M_SETTINGS_VIEW_CHANGED));
	m_pcHLayout->AddChild(m_pcIconView);
	m_pcHLayout->AddChild(new os::HLayoutSpacer("",15,15));
	pcSettingsView = new os::LayoutView(GetBounds(),"settings_view",NULL,os::CF_FOLLOW_ALL);
	pcSettingsView->SetRoot(m_cSettingNodes[0]);
	m_pcHLayout->AddChild(pcSettingsView);

	/*turn cancel and ok around to form cancel/ok*/
	/*add an export button*/
	os::HLayoutNode* pcButtonLayout = new os::HLayoutNode("buttons");
	pcButtonLayout->SetVAlignment(os::ALIGN_RIGHT);
	pcButtonLayout->SetHAlignment(os::ALIGN_RIGHT);
	pcButtonLayout->AddChild(m_pcCancel = new os::Button(os::Rect(),"Cancel_BUT","_Cancel",new os::Message(M_BUTTON_CANCEL)));
	pcButtonLayout->AddChild(new os::HLayoutSpacer("",5,5));
	pcButtonLayout->AddChild(m_pcOk = new os::Button(os::Rect(),"Save_BUT","_Save",new os::Message(M_BUTTON_OK)));
	pcButtonLayout->AddChild(new os::HLayoutSpacer("",5,5));
	pcButtonLayout->AddChild(new os::HLayoutSpacer("",5,5));
	pcButtonLayout->SameWidth("Save_BUT","Cancel_BUT",NULL);
	pcButtonLayout->SameHeight("Save_BUT","Cancel_BUT",NULL);
	pcButtonLayout->LimitMaxSize(os::Point(pcButtonLayout->GetPreferredSize(false).x+35,pcButtonLayout->GetPreferredSize(false).y));

	m_pcRoot->AddChild(m_pcHLayout);
	m_pcRoot->AddChild(new os::VLayoutSpacer("",5,5));
	m_pcRoot->AddChild(pcButtonLayout);
	m_pcRoot->AddChild(new os::VLayoutSpacer("",5,5));	
	m_pcView->SetRoot(m_pcRoot);
	
	AddChild(m_pcView);
	ResizeTo(os::Point(m_pcView->GetPreferredSize(false).x,m_pcView->GetPreferredSize(false).y));

	m_pcIconView->SetIconSelected(0,true);
}

bool SettingsDialog::OkToQuit()
{
	Message pcMsg = Message( M_SETTINGS_CLOSED );
	m_pcTarget->SendMessage(&pcMsg);
	return false;
}

void SettingsDialog::HandleMessage(os::Message* pcMessage)
{
	switch (pcMessage->GetCode())
	{
		case M_BUTTON_OK:
		{
			for (uint i=0; i<m_cSettingNodes.size(); i++)
			{
				m_cSettingNodes[i]->SaveSettings();
			}
			Message pcMsg = Message( M_SETTINGS_CLOSED );
			pcMsg.AddBool("save",true);
			m_pcTarget->SendMessage(&pcMsg);
			break;
		}
		
		case M_BUTTON_CANCEL:
		{
			OkToQuit();			
			break;
		}
		
		case M_SETTINGS_VIEW_CHANGED:
		{

			for (uint i=0; i<m_pcIconView->GetIconCount(); i++)
			{
				if (m_pcIconView->GetIconSelected(i))
				{
					pcSettingsView->SetRoot(m_cSettingNodes[i]);
					break;
				}
			}		
			break;
		}
		
		case M_SETTINGS_SOURCE_CHANGED:
		{
			DefaultAppSettings* settings = (DefaultAppSettings*)m_cSettingNodes[1];
			settings->SourceChanged();
			break;
		}
	}
}












