#include "utils.h"

#define min( a, b ) (((a) < (b)) ? (a) : (b))

using namespace os;

/************************************************
* Description: Loads a bitmapimage from resource
* Author: Rick Caudill
* Date: Thu Mar 18 21:32:37 2004
*************************************************/
os::BitmapImage* LoadImageFromResource( const os::String &cName )
{
	os::BitmapImage * pcImage = new os::BitmapImage();
	os::Resources cRes( get_image_id() );
	os::ResStream * pcStream = cRes.GetResourceStream( cName.c_str() );
	if( pcStream == NULL )
	{
		throw( os::errno_exception("") );
	}
	pcImage->Load( pcStream );
	return pcImage;
}



Bitmap* load_bitmap( StreamableIO* pcStream )
{
    Translator* pcTrans = NULL;

    TranslatorFactory* pcFactory = new TranslatorFactory;
    pcFactory->LoadAll();
    
    Bitmap* pcBitmap = NULL;
    
    try {
	uint8  anBuffer[8192];
	uint   nFrameSize = 0;
	int    x = 0;
	int    y = 0;
	
	BitmapFrameHeader sFrameHeader;
	for (;;) {
	    int nBytesRead = pcStream->Read( anBuffer, sizeof(anBuffer) );

	    if (nBytesRead == 0 ) {
		break;
	    }

	    if ( pcTrans == NULL ) {
		int nError = pcFactory->FindTranslator( "", TRANSLATOR_TYPE_IMAGE, anBuffer, nBytesRead, &pcTrans );
		if ( nError < 0 ) {
		    return( NULL );
		}
	    }
	    
	    pcTrans->AddData( anBuffer, nBytesRead, nBytesRead != sizeof(anBuffer) );

	    while( true ) {
	    if ( pcBitmap == NULL ) {
		BitmapHeader sBmHeader;
		if ( pcTrans->Read( &sBmHeader, sizeof(sBmHeader) ) != sizeof( sBmHeader ) ) {
		    break;
		}
		pcBitmap = new Bitmap( sBmHeader.bh_bounds.Width() + 1, sBmHeader.bh_bounds.Height() + 1, CS_RGB32 );
	    }
	    if ( nFrameSize == 0 ) {
		if ( pcTrans->Read( &sFrameHeader, sizeof(sFrameHeader) ) != sizeof( sFrameHeader ) ) {
		    break;
		}
		x = sFrameHeader.bf_frame.left * 4;
		y = sFrameHeader.bf_frame.top;
		nFrameSize = sFrameHeader.bf_data_size;
	    }
	    uint8 pFrameBuffer[8192];
	    nBytesRead = pcTrans->Read( pFrameBuffer, min( nFrameSize, sizeof(pFrameBuffer) ) );
	    if ( nBytesRead <= 0 ) {
		break;
	    }
	    nFrameSize -= nBytesRead;
	    uint8* pDstRaster = pcBitmap->LockRaster();
	    int nSrcOffset = 0;
	    for ( ; y <= sFrameHeader.bf_frame.bottom && nBytesRead > 0 ; ) {
		int nLen = min( nBytesRead, sFrameHeader.bf_bytes_per_row - x );
		memcpy( pDstRaster + y * pcBitmap->GetBytesPerRow() + x, pFrameBuffer + nSrcOffset, nLen );
		if ( x + nLen == sFrameHeader.bf_bytes_per_row ) {
		    x = 0;
		    y++;
		} else {
		    x += nLen;
		}
		nBytesRead -= nLen;
		nSrcOffset += nLen;
	    }
	    }
	}
	return( pcBitmap );
    }
    catch( ... ) {
	return( NULL );
    }
}


os::String GetDefaultPath()
{
	os::String cDefaultPath;
	Directory cDir( open_image_file( IMGFILE_BIN_DIR ) );
	cDir.GetPath( &cDefaultPath );
	cDefaultPath = os::String ( "file:" ) + cDefaultPath + os::String ( "/docs/startup.html" );	
	return cDefaultPath;
}

os::String GetHomePath()
{
	return getenv("HOME");
}

os::String GetBookMarkPath()
{
	os::String cReturnPath = GetHomePath() + "/Bookmarks";
	return cReturnPath;
}	

os::String GetBookMarkToolBarPath()
{
	os::String cReturnPath = GetHomePath() + "/Bookmarks/ToolBar";
	return cReturnPath;
}

os::String GetSettingsPath()
{
	os::String cReturnPath = GetHomePath() + "Settings/Abrowse/Settings";
	return cReturnPath;
}


